<?PHP
//Attribution Class .1
//This implements an attribution class for the Pool (like a person, but for references)
//Requires the db_init file
//require_once('../pool_init.php');
require_once('db_init.php');

class Pool_Attribution{
	//Public variables (from db)
	public $id = false;						//ID of the attrbituion
	public $hr_id = '';						//Human-readable ID of the attribution
	public $name = '';						//Written name of the attribution
	
	public $ratings = array();				//Placeholder for the ratings for this attribution
	//Protected db object
	//This is filled in the constructor
	protected $data = '';
	
	//Public functions
	public function __construct($construct){
		//If $construct is a string, this function not only constructs the object
		//  but also creates a new entry in the attribution table.
		//If $construct is an integer, it is assumed to be an attribution id
		
		//Establish the database connection
		$this->data = &$GLOBALS['data'];
		
		if(is_numeric($construct)) $this->load_id($construct);
		else if(is_string($construct)) $this->create_attribution($construct);
		else return false;
		
		if($this->id == false) return false;
	}
	
	public function load_id($attribution_id){
		//Loads data from the attribution table based on the numeric id
		
		$this->id = $attribution_id;
		
		//Get the person row
		$q = "SELECT * FROM `attribution` WHERE `id` = ?";
		$result = $this->data->getRow($q, array($this->id));
		if(!$this->check_db($result, 'load_id')) return false;
		foreach($result as $key=>$col) {
			$this->$key = $col;
		}
		return true;
	}
	
	public function load_hr_id($hr_id){
		//Loads data from the attribution table based on the hr id
		$this->id = $this->resolve_hr($hr_id);
		$this->load_id($this->id);
	}
	
	public function load_review_js(){
		//Loads js_cache objects for the reviews involving this attribution
		//Creates $this->reviews_of
		if($this->id == '') return false;
		
		$q = 'SELECT `review`.`js_cache`, `review`.`id` 
				FROM `review`, `reference_attribution`
				WHERE `reference_attribution`.`attribution_id` = ? AND
					  `review`.`artifact_type` = ? AND
					`reference_attribution`.`reference_id` = `review`.`artifact_id`
				ORDER BY `review`.`date` DESC';
		$result = $this->data->getAll($q, array($this->id, 'reference'));
		if(!$this->check_db($result, 'load_review_js')) return false;
		$this->reviews_of = $result;
	}
	
	public function load_reference_js(){
		//Loads the js_cache_short objects for the references attributed to this author
		//Creates $this->references
		if($this->id == '') return false;
		
		$q = 'SELECT `reference`.`js_cache_short`
				FROM `reference`, `reference_attribution`
				WHERE `reference_attribution`.`attribution_id` = ?
					AND `reference`.`id` = `reference_attribution`.`reference_id`
				GROUP BY `reference`.`id`
				ORDER BY `reference`.`date_published` DESC';
		$result = $this->data->getAll($q, array($this->id));
		if(!$this->check_db($result, 'load_reference_js')) return false;
		$this->references = $result;		
	}
	
	public function load_ratings(){
	//Loads this attribution's ratings into $this->ratings
		if($this->id == '') return false;
		
		$q = 'SELECT `category`, `rating`, `basis` FROM `attribution_ratings` WHERE `attribution_id` = ?';
		$result = $this->data->getAll($q, array($this->id));
		if(!$this->check_db($result, 'load_ratings')) return false;
		foreach($result as $rating){
			$this->ratings[$rating['category']] = array('rating' => $rating['rating'], 'basis' => $rating['basis']);
		}
	}
	
	public function create_attribution($name){
		//Create a new attribution and enters them into the database.	

		//Create an hr_id
		$hr_id = Pool_Attribution::name_to_hr_id($name);
		
		//Add the attribution to the db
		$q = 'INSERT INTO `attribution` (`name`, `hr_id`) VALUES (?,?)';
		$result = $this->data->query($q, array($name, $hr_id));
		if(!$this->check_db($result, 'create_attribution')) return false;
		
		//Set up object variables
		$this->id = Pool_Attribution::resolve_hr($hr_id);
		$this->hr_id = $hr_id;
		$this->name = $args['name'];
	}
	
	public function get_review_subratings(){
		//Gets the subratings of all references this person has contributed to
		$q = 'SELECT `review_ratings`.`category`, `review_ratings`.`rating`, `review`.`recorder_id`
				FROM `review`, `reference_attribution`, `review_ratings`
				WHERE `review`.`artifact_id` = `reference_attribution`.`reference_id`
					AND `review`.`artifact_type` = ?
					AND `review`.`id` = `review_ratings`.`review_id`
					AND `reference_attribution`.`attribution_id` = ?
					GROUP BY `review_ratings`.`category`, `review_ratings`.`rating`, `review`.`recorder_id`';
		$result = $this->data->getAll($q, array('reference', $this->id));
		if(!$this->check_db($result, 'get_reviews')) return false;
		$subratings = array();
		foreach($result as $subrating){
			if(!isset($subratings[$subrating['recorder_id']])) $subratings[$subrating['recorder_id']] = array();
			$subratings[$subrating['recorder_id']][$subrating['category']] = $subrating['rating'];
		}
		
		return $subratings;		
	}
	
	public function get_reviewer_creds(){
		//get the credibilities for everybody who has written reviews of this attribution's references
		
		$q = 'SELECT `person_ratings`.`person_id`, `person_ratings`.`category`, `person_ratings`.`rating`
				FROM `person_ratings`, `review`, `reference_attribution`
				WHERE `review`.`artifact_id` = `reference_attribution`.`reference_id`
					AND `review`.`artifact_type` = ?
					AND `review`.`recorder_id` = `person_ratings`.`person_id`
					AND `reference_attribution`.`attribution_id` = ?
					GROUP BY `person_ratings`.`person_id`, `person_ratings`.`category`, `person_ratings`.`rating`';
		$result = $this->data->getAll($q, array('reference', $this->id));
		if(!$this->check_db($result, 'get_reviewer_creds')) return false;
		$creds = array();
		foreach($result as $cred){
			if(!isset($creds[$cred['person_id']])) $creds[$cred['person_id']] = array();
			$creds[$cred['person_id']][$cred['category']] = $cred['rating'];
		}
		
		return $creds;
	}
	
	public function set_rating_and_basis(){
		//Sets the rating and basis for this person
		
		//First get all the reviews for artifacts this person has contributed to
		$subratings = $this->get_review_subratings();
		
		//Get all the credibilities for the reviewers
		$creds = $this->get_reviewer_creds();

		//Apply credibilities to review ratings
		//$ratings is an array of rating categories.  Each category has a weight and a total.  Weight is the sum of credibilities, while total is the sum of (credibilities * ratings)
		$ratings = array();
		
		foreach($subratings as $person_id => $categories){
			foreach($categories as $category => $rating){
				//Check to see if this reviewer has a credibility for this category of rating.  If it does not exist, set it to 1
				if(!isset($creds[$person_id][$category])) $creds[$person_id][$category] = 1;
				//If this is the first rating of the category, initialize it in the ratings array
				if(!isset($ratings[$category])) $ratings[$category] = array('weight' => 0, 'total' => 0, 'basis' => 0);
				//Add the rating values
				$ratings[$category]['weight'] += $creds[$person_id][$category];
				$ratings[$category]['total'] += $creds[$person_id][$category] * $rating;
				$ratings[$category]['basis']++;
			}
		}

		//Calculate the weighted ratings
		foreach($ratings as &$subrating){
			if($subrating['weight'] == 0) $subrating['final'] = $subrating['total'];
			else $subrating['final'] = $subrating['total'] / $subrating['weight'];
		}
		
		//Store the new rating, subratings, and basis
		foreach($ratings as $category => $subrating){
			$values_arr = array();
			$data_arr = array();
			$values_arr[] = '(?, ?, ?, ?, ?)';
			$data_arr[] = $this->id;
			$data_arr[] = $category;
			$data_arr[] = $subrating['final'];
			$data_arr[] = $subrating['basis'];
			$data_arr[] = $this->hr_id.'_'.$category;

			$values_str = implode(',', $values_arr);
			
			$q = "INSERT INTO `attribution_ratings` (`attribution_id`, `category`, `rating`, `basis`, `hr_id`)
					VALUES $values_str ON DUPLICATE KEY UPDATE `rating` = VALUES(`rating`), `basis` = VALUES(`basis`)";
			$result = $this->data->query($q, $data_arr);
			if(!$this->check_db($result, 'set_rating_and_basis')) return false;
		}
		
		//update the js_cache for the person
		$this->reload_all();
		$this->save_js();

		return true;
	}
	
	public function reload_all(){
	//reloads person data from the db
		$this->load_id($this->id);
		$this->load_review_js();
		$this->load_reference_js();
		$this->load_ratings();
	}
	
	public function save_js(){
		//saves the js_cache for this person
		require_once('smarty_init.php');
		$smarty = &$GLOBALS['smarty'];
		
		$this->reload_all();
		
		$smarty->assign('attribution', $this);
		$smarty->left_delimiter = '<{';
		$smarty->right_delimiter = '}>';
		$js_cache = $smarty->fetch('js_attribution_array.tpl');
		$smarty->left_delimiter = '{';
		$smarty->right_delimiter = '}';
		
		$q = 'UPDATE `attribution` SET `js_cache` = ? WHERE `id` = ?';
		$this->data->query($q, array($js_cache, $this->id));
	}
	
	public function write_js(){
		//writes the js_cache for this person
		require_once('smarty_init.php');
		$smarty = &$GLOBALS['smarty'];
		
		$this->reload_all();
		
		$smarty->assign('attribution', $this);
		$smarty->left_delimiter = '<{';
		$smarty->right_delimiter = '}>';
		$js_cache = $smarty->display('js_attribution_array.tpl');
		$smarty->left_delimiter = '{';
		$smarty->right_delimiter = '}';	
	}	
	
	//protected functions	
	protected function check_db($result, $func){
		//Return true if $result is not DB::Error, false and an error otherwise
		if(DB::isError($result)){
			trigger_error("Pool_Attribution->$func failed: \n".$this->data->last_query."\n".$result->getMessage(), E_USER_WARNING);	
			return false;
		} else return true;
	}

	
	//Static functions	
	public static function name_to_hr_id($name){
		//Creates a new hr_id from a written name.
				
		//Get the database
		$data = &$GLOBALS['data'];
		
		//Build the text part of the name
		$name_split = explode(' ', $name);
		$pool_name = implode('', $name_split).substr($lname.$fname, 0, 30);
		
		$escaped = $data->escapeSimple($pool_name).'*';
		
		//Find out if this name already exists in the db
		$q = "SELECT COUNT(*) as count FROM `attribution` WHERE `hr_id` LIKE '$escaped'";
		$result = (int) $data->getOne($q, array());
		$result++;
		$pool_name .= sprintf("%02d", $result);
		return $pool_name;
	}
	
	public static function resolve_hr($hr_id){
	//Turns an hr_id into a numeric one
	
		$data = &$GLOBALS['data'];
		$q = "SELECT `id` FROM `attribution` WHERE `hr_id` = ?";
		$result = $data->getOne($q, array($hr_id));
		if(DB::isError($result)){
			trigger_error("Could not resolve hr_id '$hr_id' in 'attribution' table.", E_USER_WARNING);
			return false;
		} else {
			return $result;
		}	
	}
	
	public static function get_all_written(){
	//Returns an array with id => [hr_id: '', written: '']

		$data = &$GLOBALS['data'];
		$q = 'SELECT `id`, `hr_id`, `name` FROM `attribution` ORDER BY `name`';
		$result = $data->getAll($q, array());
		if(DB::isError($result)){
			trigger_error("Error retrieving data from attribution table in attribution->get_all_written.", E_USER_WARNING);
			return false;
		} else {
			return $result;
		}
	}
	
	public static function get_name($attribution_id){
	//Returns a single written name for the passed person id
		
		$data = &$GLOBALS['data'];
		$q = 'SELECT `name` FROM `attribution` WHERE `id` = ?';
		$result = $data->getOne($q, array($attribution_id));
		if(DB::isError($result)){
			trigger_error("Error retrieving data from attribution table in attribution->get_name.", E_USER_WARNING);
			return false;
		} else {
			return $result;
		}
	}
	
	public static function get_hr_id($attribution_id){
	//Returns a single hr_id for the passed attribution id
		
		$data = &$GLOBALS['data'];
		$q = 'SELECT `hr_id` FROM `attribution` WHERE `id` = ?';
		$result = $data->getOne($q, array($person_id));
		if(DB::isError($result)){
			trigger_error("Error retrieving data from attribution table in attribution->get_hr_id.", E_USER_WARNING);
			return false;
		} else {
			return $result;
		}	
	}
	
	
	public static function get_cached_js($person_id){
	//writes the js_cache for the passed person ID to stdout
		$data = &$GLOBALS['data'];
		
		$q = 'SELECT `js_cache` FROM `attribution` WHERE `id` = ?';
		$result = $data->getOne($q, array($person_id));
		if(DB::isError($result)){
			trigger_error("Error retrieving js cache from person table in person->get_cached_js.", E_USER_WARNING);
			return false;
		} else {
			echo $result;
		}
	}
}
?>