<?PHP
/** 
* Error Handling Class (http://www.phpfreaks.com/tutorials/117/0.php)
* 
* @author Derek Ford <derek at php dot net> 
* @version $Revision: 1.1 $ 
* @since 1.0 
* @package packages?wedon'tneednostinkin'packages! 
* @access public 
*/ 
class clsErrorHandle { 
    /** 
    * The filename of the log file. 
    * 
    * NOTE: $error_log_filename will only be used if you have log_errors Off and ;error_log filename in php.ini 
    * if log_errors is On, and error_log is set, the filename in error_log will be used. 
    * 
    * @var string 
    * @access public 
    */ 
    var $error_log_filename; 
    /** 
    * The recipient email to mail errors to 
    * 
    * @var string 
    * @access public 
    */ 
    var $to; 
    /** 
    * Holds the total error report to be used by mailError() 
    * 
    * @var string 
    * @access public 
    */ 
    var $mail_buffer; 
    /** 
    * The flag that determines whether or not warnings will be logged to the error file. 
    * 
    * @var Boolean 
    * @access public 
    * @see $LOG_ERRORS_TO_FILE 
    * @see $LOG_NOTICES_TO_FILE 
    */ 
    var $LOG_WARNINGS_TO_FILE; 
    /** 
    * The flag that determines whether or not notices will be logged to the error file. 
    * 
    * @var Boolean 
    * @access public 
    * @see $LOG_ERRORS_TO_FILE 
    * @see $LOG_WARNINGS_TO_FILE 
    */ 
    var $LOG_NOTICES_TO_FILE; 
    /** 
    * Send warnings via email? 
    * 
    * @var Boolean 
    * @access public 
    * @see $SEND_ERRORS_TO_MAIL; 
    * @see $SEND_NOTICES_TO_MAIL; 
    */ 
    var $SEND_WARNINGS_TO_MAIL; 
    /** 
    * Send notices via email? 
    * 
    * @var Boolean 
    * @access public 
    * @see $SEND_ERRORS_TO_MAIL; 
    * @see $SEND_WARNINGS_TO_MAIL 
    */ 
    var $SEND_NOTICES_TO_MAIL; 
    /** 
    * Send Errors/Warnings/Notices to output? 
    * 
    * @var bool 
    * @access public 
    */ 
    var $SEND_ERRORS_TO_OUTPUT; 
    /** 
    * clsErrorHandle::clsErrorHandle 
    * 
    * @return void 
    * @access public 
    */ 
    function clsErrorHandle() { 
        $this->sendMailTo('Whomever', 'whomever@whatever.com'); 
        $this->setFilename(''); 
        if (isset($_GET['showerrors'])) 
            $this->setFlags(false, false, false, false, true); //This will ONLY output errors to STDOUT. 
        else 
            $this->setFlags(false, false, true, true, false); //This will not log errors to file, but will send error reports via email. 
        set_error_handler(array(&$this, 'ERROR_HANDLER')); 
        register_shutdown_function(array(&$this, 'clsErrorHandleDestruct')); 
    } 
    /** 
    * clsErrorHandle::clsErrorHandleDestruct 
    * 
    * @return void 
    * @access public 
    */ 
    function clsErrorHandleDestruct() { 
        if (strlen($this->mail_buffer) > 0) 
            $this->mailError($this->mail_buffer); //Send the email 
    } 
    /** 
    * The error handling routine set by set_error_handler() 
    * 
    * @param string $error_type The type of error being handled. 
    * @param string $error_msg The error message being handled. 
    * @param string $error_file The file in which the error occurred. 
    * @param integer $error_line The line in which the error occurred. 
    * @param string $error_context The context in which the error occurred. 
    * @return Boolean 
    * @access public 
    * @see clsErrorHandle() 
    */ 
    function ERROR_HANDLER($error_type, $error_msg, $error_file, $error_line, $error_context) { 
        switch ($error_type) { 
            case E_WARNING: 
            case E_USER_WARNING: 
                if ($this->LOG_WARNINGS_TO_FILE) { 
                    if ($this->error_log_filename == '') 
                        error_log($error_msg . ' (error type ' . $error_type . ' in ' . $error_file . ' on line ' . $error_line . ')' . chr(10), 0); 
                    else 
                        error_log($error_msg . ' (error type ' . $error_type . ' in ' . $error_file . ' on line ' . $error_line . ')' . chr(10), 3, $this->error_log_filename); 
                } 
                if ($this->SEND_WARNINGS_TO_MAIL) 
                    $this->mail_buffer .= $error_msg . ' (error type ' . $error_type . ' in ' . $error_file . ' on line ' . $error_line . ')' . chr(10); 
                if ($this->SEND_ERRORS_TO_OUTPUT) 
                    echo $error_msg, ' (error type ', $error_type, ' in ', $error_file, ' on line ', $error_line, ')<br />'; 
                break; 
            case E_NOTICE: 
            case E_USER_NOTICE: 
                if (substr_count($error_msg, ';') == 1) { 
                    $error = explode(';', $error_msg); 
                    switch($error[0]) { 
                        case 'dberror': 
                            // trigger_error('dberror;error message', E_USER_NOTICE) 
                            break; 
                        default: 
                            echo $error[1], ' oh noes!'; 
                    } 
                }    else    { 
                    if ($this->LOG_NOTICES_TO_FILE) { 
                        if ($this->error_log_filename == '') 
                            error_log($error_msg . ' (error type ' . $error_type . ' in ' . $error_file . ' on line ' . $error_line . ')' . chr(10), 0); 
                        else 
                            error_log($error_msg . ' (error type ' . $error_type . ' in ' . $error_file . ' on line ' . $error_line . ')' . chr(10), 3, $this->error_log_filename); 
                    } 
                    if ($this->SEND_NOTICES_TO_MAIL) 
                         $this->mail_buffer .= $error_msg . ' (error type ' . $error_type . ' in ' . $error_file . ' on line ' . $error_line . ')' . chr(10); 
                    if ($this->SEND_ERRORS_TO_OUTPUT) 
                        echo $error_msg, ' (error type ', $error_type, ' in ', $error_file, ' on line ', $error_line, ')<br />'; 
                 } 
                break; 
        } 
        return true; 
    } //END OF ERROR_HANDLER() 
    /** 
    * Method that changes the filename of the generated log file. 
    * 
    * @param string $filename The filename to be used for the log file. 
    * @return void 
    * @access public 
    * @see clsErrorHandle() 
    */ 
    function setFilename($filename) { 
        $this->error_log_filename = $filename; //Error Log Filename 
    }//END OF setFilename() 
    /** 
    * Method that changes the logging flags.. 
    * 
    * @param Boolean $warning_flag Log warnings to file? 
    * @param Boolean $notice_flag Log notices to file? 
    * @param Boolean $warning_mailflag Send warnings via mail? 
    * @param Boolean $notice_mailflag Send notices via mail? 
    * @return void 
    * @access public 
    * @see clsErrorHandle() 
    */ 
    function setFlags($warning_flag = true, $notice_flag = true, $warning_mailflag = true, $notice_mailflag = true, $error_output = true) { 
        $this->LOG_WARNINGS_TO_FILE = $warning_flag; 
        $this->LOG_NOTICES_TO_FILE = $notice_flag; 
        $this->SEND_WARNINGS_TO_MAIL = $warning_mailflag; 
        $this->SEND_NOTICES_TO_MAIL = $notice_mailflag; 
        $this->SEND_ERRORS_TO_OUTPUT = $error_output; 
    }//END OF setFlags() 
    /** 
    * Method that restores the error handler to the default error handler 
    * 
    * @return void 
    * @access public 
    * @see clsErrorHandle() 
    */ 
    function restoreHandler() { 
        restore_error_handler(); 
    } 
    /** 
    * Method that returns the error handler to ERROR_HANDLER() 
    * 
    * @return void 
    * @access public 
    * @see clsErrorHandle() 
    */ 
    function returnHandler() { 
        set_error_handler(array(&$this, 'ERROR_HANDLER')); 
    } 
    /** 
    * This will print the associative array populated by backtrace data 
    * 
    * @return void 
    * @access public 
    * @see clsErrorHandle() 
    */ 
    function showBacktrace() { 
        debug_print_backtrace(); 
    } 
    /** 
    * Method that is used to send error reports via email 
    * 
    * @param string $mail_body Error message 
    * @return void 
    * @access public 
    * @see clsErrorHandle() 
    */ 
    function mailError($mail_body) { 
        $headers = 'From: "Error Handler" <pool.error.logs@umit.maine.edu>' . "\r\n"; 
        $headers .= 'MIME-Version: 1.0' . "\r\n"; 
        $headers .= 'Content-type: text/plain; charset=iso-8859-1' . "\r\n"; 
        mail($this->to, 'Error Report', $mail_body, $headers); 
    } 
    /** 
    * This method will setup php to send mail via mail() on a Windows server 
    * 
    * @param string $smtp_server Your SMTP Server address 
    * @param string $sendmail_from Your email address 
    * @return void 
    * @access public 
    * @see clsErrorHandle() 
    */ 
    function setupWinMail($smtp_server, $sendmail_from) { 
        ini_set('SMTP', $smtp_server); 
        ini_set('sendmail_from', $sendmail_from); 
    } 
    /** 
    * This method will setup php to send mail via mail() on a Linux server 
    * 
    * Usually /usr/bin/sendmail 
    * 
    * @param string $sendmailpath Path to sendmail on your server, and whatever flags you wish to use with sendmail. 
    * @return void 
    * @access public 
    * @see clsErrorHandle() 
    */ 
    function setupLinMail($sendmailpath) { 
        ini_set('sendmail_path', $sendmailpath); 
    } 
    /** 
    * This method will set which email address error reports are sent to. 
    * 
    * @param string $recipient_name The name of the recipient. 
    * @param string $recipient_address The email address of the recipient. 
    * @return void 
    * @access public 
    * @see clsErrorHandle() 
    */ 
    function sendMailTo($recipient_name, $recipient_address) { 
        $this->to = $recipient_name . ' <' . $recipient_address . '>'; 
    } 
} 
?>