<?PHP
//Person Class .2
//This implements person class for the Pool
//Requires the db_init file
//require_once('../pool_init.php');
require_once('db_init.php');

class Pool_Person{
	//Public variables (from db)
	public $id = false;						//ID of the person
	public $hr_id = '';						//Human-readable ID of the person
	public $email = '';						//email address of the person
	public $uri = '';						//URI associated with the person
	public $description = '';				//Description of the person
	public $themes = '';					//Themes associated with the person
	public $last_login = '';				//Datetime of the person's last login
	public $recorder_id = '';				//Person ID of the recorder for the person
	public $thumb_id = '';					//ID of the person's thumbnail image in the thumbs table
	public $written = '';					//The person's written name
	public $permissions = '';				//Permissions granted to this person
											//Right now, 1=user, 2=admin.  This is subject to change.
	public $ratings = array();				//Placeholder for the ratings for this person
	//Protected db object
	//This is filled in the constructor
	protected $data = '';
	
	//Public functions
	public function __construct($construct){
		//If $construct is an array, this function not only constructs the object
		//  but also creates a new entry in the person table.
		//If $construct is an integer, it is assumed to be a person id
		//If $construct is a string, it is assumed to be a person's hr_id
		
		//Establish the database connection
		$this->data = &$GLOBALS['data'];
		
		if(is_array($construct)) $this->create_user($construct);
		else if(is_numeric($construct)) $this->load_id($construct);
		else if(is_string($construct)) $this->load_hr_id($construct);
		else return false;
		
		if($this->id == false) return false;
	}
	
	public function load_id($person_id){
		//Loads data from the person table based on the numeric id
		
		$this->id = $person_id;
		
		//Get the person row
		$q = "SELECT * FROM `person` WHERE `id` = ?";
		$result = $this->data->getRow($q, array($this->id));
		if(!$this->check_db($result, 'load_id')) return false;
		foreach($result as $key=>$col) {
			if($key != 'password')	$this->$key = $col;
		}
		return true;
	}
	
	public function load_hr_id($hr_id){
		//Loads data from the person table based on the hr id
		$this->id = $this->resolve_hr($hr_id);
		$this->load_id($this->id);
	}
	
	public function load_review_js(){
		//Loads js_cache objects for the reviews involving this person
		//Creates $this->reviews_by and $this->reviews_of
		if($this->id == '') return false;
		
		$q = 'SELECT `review`.`js_cache`, `review`.`id` 
				FROM `review`, `person_artifact`
				WHERE `person_artifact`.`person_id` = ? AND
					  `person_artifact`.`artifact_type` = `review`.`artifact_type` AND
					`person_artifact`.`artifact_id` = `review`.`artifact_id`
				ORDER BY `review`.`date` DESC';
		$result = $this->data->getAll($q, array($this->id));
		if(!$this->check_db($result, 'load_review_js')) return false;
		$this->reviews_of = $result;
		
		$q = 'SELECT `review`.`js_cache`, `review`.`id`
				FROM `review`
				WHERE `review`.`recorder_id` = ?
				ORDER BY `review`.`date` DESC';
		$result = $this->data->getAll($q, array($this->id));
		if(!$this->check_db($result, 'load_review_js')) return false;
		$this->reviews_by = $result;
	}
	
	public function load_project_js(){
		//Loads the js_cache_short objects for the projects this person is a contributor to
		//Creates $this->projects
		if($this->id == '') return false;
		
		$q = 'SELECT `project`.`js_cache_short`
				FROM `project`, `project_artifact`, `person_artifact`
				WHERE `person_artifact`.`person_id` = ?
					AND `person_artifact`.`artifact_type` = `project_artifact`.`artifact_type`
					AND `project_artifact`.`artifact_id` = `person_artifact`.`artifact_id`
					AND `project`.`id` = `project_artifact`.`project_id`
				GROUP BY `project`.`id`
				ORDER BY `project`.`last_modified` DESC';
		$result = $this->data->getAll($q, array($this->id));
		if(!$this->check_db($result, 'load_project_js')) return false;
		$this->projects = $result;		
	}
	
	public function load_ratings(){
	//Loads this person's ratings into $this->ratings
		if($this->id == '') return false;
		
		$q = 'SELECT `category`, `rating`, `basis`, `self_rating` FROM `person_ratings` WHERE `person_id` = ?';
		$result = $this->data->getAll($q, array($this->id));
		if(!$this->check_db($result, 'load_ratings')) return false;
		foreach($result as $rating){
			$this->ratings[$rating['category']] = array('rating' => $rating['rating'], 'basis' => $rating['basis'], 'self_rating' => $rating['self_rating']);
		}
	}
	
	public function create_user($args){
		//Create a new user and enters them into the database.	
		//Note that this assumes multiple users with the same name can exist,
		// so duplicates are not trapped
		//$args array must have:
		//	name (written name)
		//	email
		//	permissions (numeric)
		//	recorder (numeric id)
		//Args array may optionally contain:
		//	password
		//	uri
		//	description
		
		//Make sure required params are actually there
		if(!isset($args['name']) || !isset($args['email']) || !isset($args['permissions']) || !isset($args['recorder'])){
			trigger_error('Could not create a new user:  A required parameter was not set', E_USER_WARNING);
			return false;	
		}
		//Create an hr_id
		$hr_id = 'per_' . Pool_Person::name_to_hr_id($args['name']);
		//If a password was not passed, create a random one.
		if(isset($args['password'])) $password = $args['password'];
		else{
			for($len=8,$password='';strlen($password)<$len;$password.=chr(!mt_rand(0,2)?mt_rand(48,57):(!mt_rand(0,1)?mt_rand(65,90):mt_rand(97,122))));			
		} 
		
		//Add the person to the db
		$q = 'INSERT INTO `person` (`written`, `email`, `recorder_id`, `permissions`, `hr_id`, `password`) VALUES (?,?,?,?,?,PASSWORD(?))';
		$result = $this->data->query($q, array($args['name'], $args['email'], $args['recorder'], $args['permissions'], $hr_id, $password));
		if(!$this->check_db($result, 'create_user')) return false;
		
		//Set up object variables
		$this->id = Pool_Person::resolve_hr($hr_id);
		$this->hr_id = $hr_id;
		$this->email = $args['email'];
		$this->written = $args['name'];
		$this->permissions = $args['permissions'];
		$this->recorder_id = $args['recorder'];
		
		//Set the optional parameters
		if(isset($args['uri'])) $this->set_value('uri', $args['uri']);
		if(isset($args['description'])) $this->set_value('description', $args['description']);
		
		//Rebuild JS array
		$this->reload_all();
		$this->save_js();
		return true;
	}
	
	public function get_review_subratings(){
		//Gets the subratings of all artfacts this person has contributed to
		$q = 'SELECT `review_ratings`.`category`, `review_ratings`.`rating`, `review`.`recorder_id`
				FROM `review`, `person_artifact`, `review_ratings`
				WHERE `review`.`artifact_id` = `person_artifact`.`artifact_id`
					AND `review`.`artifact_type` = `person_artifact`.`artifact_type`
					AND `review`.`id` = `review_ratings`.`review_id`
					AND `person_artifact`.`person_id` = ?
					GROUP BY `review_ratings`.`category`, `review_ratings`.`rating`, `review`.`recorder_id`';
		$result = $this->data->getAll($q, array($this->id));
		if(!$this->check_db($result, 'get_reviews')) return false;
		$subratings = array();
		foreach($result as $subrating){
			if(!isset($subratings[$subrating['recorder_id']])) $subratings[$subrating['recorder_id']] = array();
			$subratings[$subrating['recorder_id']][$subrating['category']] = $subrating['rating'];
		}
		
		return $subratings;		
	}
	
	public function get_reviewer_creds(){
		//get the credibilities for everybody who has written reviews of this person's artifacts
		
		$q = 'SELECT `person_ratings`.`person_id`, `person_ratings`.`category`, `person_ratings`.`rating`
				FROM `person_ratings`, `review`, `person_artifact`
				WHERE `review`.`artifact_id` = `person_artifact`.`artifact_id`
					AND `review`.`artifact_type` = `person_artifact`.`artifact_type`
					AND `review`.`recorder_id` = `person_ratings`.`person_id`
					AND `person_artifact`.`person_id` = ?
					GROUP BY `person_ratings`.`person_id`, `person_ratings`.`category`, `person_ratings`.`rating`';
		$result = $this->data->getAll($q, array($this->id));
		if(!$this->check_db($result, 'get_reviewer_creds')) return false;
		$creds = array();
		foreach($result as $cred){
			if(!isset($creds[$cred['person_id']])) $creds[$cred['person_id']] = array();
			$creds[$cred['person_id']][$cred['category']] = $cred['rating'];
		}
		
		return $creds;
	}
	
	public function set_rating_and_basis(){
		//Sets the rating and basis for this person
		
		//First get all the reviews for artifacts this person has contributed to
		$subratings = $this->get_review_subratings();
		
		//Get all the credibilities for the reviewers
		$creds = $this->get_reviewer_creds();

		//Apply credibilities to review ratings
		//$ratings is an array of rating categories.  Each category has a weight and a total.  Weight is the sum of credibilities, while total is the sum of (credibilities * ratings)
		$ratings = array();
		
		foreach($subratings as $person_id => $categories){
			foreach($categories as $category => $rating){
				//Check to see if this reviewer has a credibility for this category of rating.  If it does not exist, set it to 1
				if(!isset($creds[$person_id][$category])) $creds[$person_id][$category] = 1;
				//If this is the first rating of the category, initialize it in the ratings array
				if(!isset($ratings[$category])) $ratings[$category] = array('weight' => 0, 'total' => 0, 'basis' => 0);
				//Add the rating values
				$ratings[$category]['weight'] += $creds[$person_id][$category];
				$ratings[$category]['total'] += $creds[$person_id][$category] * $rating;
				$ratings[$category]['basis']++;
			}
		}

		//Calculate the weighted ratings
		foreach($ratings as &$subrating){
			$subrating['final'] = $subrating['total'] / $subrating['weight'];
		}
		
		//Store the new rating, subratings, and basis
		foreach($ratings as $category => $subrating){
			$values_arr = array();
			$data_arr = array();
			$values_arr[] = '(?, ?, ?, ?, ?)';
			$data_arr[] = $this->id;
			$data_arr[] = $category;
			$data_arr[] = $subrating['final'];
			$data_arr[] = $subrating['basis'];
			$data_arr[] = $this->hr_id.'_'.$category;

			$values_str = implode(',', $values_arr);
			
			$q = "INSERT INTO `person_ratings` (`person_id`, `category`, `rating`, `basis`, `hr_id`)
					VALUES $values_str ON DUPLICATE KEY UPDATE `rating` = VALUES(`rating`), `basis` = VALUES(`basis`)";
			$result = $this->data->query($q, $data_arr);
			if(!$this->check_db($result, 'set_rating_and_basis')) return false;
		}
		
		//update the js_cache for the person
		$this->reload_all();
		$this->save_js();

		return true;
	}
	
	public function save_js(){
		//saves the js_cache for this person
		require_once('smarty_init.php');
		$smarty = &$GLOBALS['smarty'];
		
		$this->reload_all();
		
		$smarty->assign('person', $this);
		$smarty->left_delimiter = '<{';
		$smarty->right_delimiter = '}>';
		$js_cache = $smarty->fetch('js_person_array_full.tpl');
		$smarty->left_delimiter = '{';
		$smarty->right_delimiter = '}';
		
		$q = 'UPDATE `person` SET `js_cache` = ? WHERE `id` = ?';
		$this->data->query($q, array($js_cache, $this->id));
	}
	
	public function write_js(){
		//writes the js_cache for this person
		require_once('smarty_init.php');
		$smarty = &$GLOBALS['smarty'];
		
		$this->reload_all();
		
		$smarty->assign('person', $this);
		$smarty->left_delimiter = '<{';
		$smarty->right_delimiter = '}>';
		$js_cache = $smarty->display('js_person_array_full.tpl');
		$smarty->left_delimiter = '{';
		$smarty->right_delimiter = '}';	
	}	
	
	public function reload_all(){
	//reloads person data from the db
		$this->load_id($this->id);
		$this->load_review_js();
		$this->load_project_js();
		$this->load_ratings();
	}

	public function set_value($col, $val){
		//Sets a value in the database and the object
		
		$q = "UPDATE `person` SET `$col` = ? WHERE `id` = ?";
		$result = $this->data->query($q, array($val, $this->id));
		if(!$this->check_db($result, 'set_value')) return false;
		else $this->$col = $val;
		return true;
	}

	public function send_info($password){
		$mime_boundary=md5(time());
		
		$subject = 'Your account information for The Pool';
		$message = "Welcome to The Pool!  Your account has been created as:\n".
					"\tUser name: ".substr($this->hr_id,4)."\n".
					"\tPassword: ".$password."\n\n".

					"Now that you have an account, there are a few things you might want to try out. (You may have to copy/paste these links for them to work)\n".
					 "\tLog in at our homepage ( http://pool.newmedia.umaine.edu/ )\n".
					 "\tRead the FAQ ( http://pool.newmedia.umaine.edu/faq.html )\n".
					 "\tCheck out the walkthrough ( http://pool.newmedia.umaine.edu/walkthrough/home.html )\n".
					 "\tBrowse the art or code pools ( http://pool.newmedia.umaine.edu/art_pool.php and http://pool.newmedia.umaine.edu/code_pool.php )\n".
					 "\tOnce you log in, set your user profile ( http://pool.newmedia.umaine.edu/art_pool.php#p=person|".$this->id." )\n".
					 "\nEnjoy!";
					 
		$headers =  "From: ".POOL_RETURN_ADDRESS."\r\n" .
   					"Reply-To: ".POOL_RETURN_ADDRESS."\r\n" .
   				//	"Return-Path: ".POOL_RETURN_ADDRESS."\r\n" .
					"Message-ID: <".time()."-".POOL_RETURN_ADDRESS.">\r\n".
   					"X-Mailer: PHP v" . phpversion() ."\r\n";
   				//	"--".$mime_boundary."\r\n".
   				//	"Content-Type: text/plain; charset=iso-8859-1\r\n".
				//	"Content-Transfer-Encoding: 8bit\r\n\r\n".
				//	"--".$mime_boundary."\r\n\r\n";

		ini_set('sendmail_from',POOL_RETURN_ADDRESS);  // the INI lines are to force the From Address to be used !
		$params = '-F'.POOL_RETURN_ADDRESS.' -f'.POOL_RETURN_ADDRESS;
		//$params = '-r '.POOL_RETURN_ADDRESS;
		$mail_sent = mail($this->email, $subject, $message, $headers, $params);
		ini_restore('sendmail_from');
	}
	
	//protected functions	
	protected function check_db($result, $func){
		//Return true if $result is not DB::Error, false and an error otherwise
		if(DB::isError($result)){
			trigger_error("Pool_Person->$func failed: $this->data->last_query \n".$result->getMessage(), E_USER_WARNING);	
			return false;
		} else return true;
	}

	
	//Static functions	
	public static function db_auth($id, $pass){
		//Does a db authentication check without creating the entire Pool_Person object
		//if $id is an int, it is assumed to be person.id
		//if $id is a string, it is assumed to be person.hr_id
		//If authentication fails, returns false
		//If authentication succeeds, returns the permissions level of the person
		
		//Get the database
		$data = &$GLOBALS['data'];
		
		//Check whether this is an id or an hr_id
		$col = is_numeric($id) ? 'id' : 'hr_id';
		$q = "SELECT COUNT(*) AS num, `permissions`, `id`, `written`, `hr_id` 
				FROM `person` 
				WHERE `person`.`$col` = ? 
					AND `person`.`password` = PASSWORD(?)
				GROUP BY `id`";
		$result = $data->getRow($q, array($id, $pass));
		if(DB::isError($result)){
			//trigger_error("Could not authenticate id '$id' in 'person' table.", E_USER_WARNING);
			return false;
		} else {
			if($result['num'] == 0) return false;
			self::update_time($result['id']);
			self::set_cookie($result['id'], $result['written'], $result['permissions']);
			return $result;
		}
	}
	
	public static function set_cookie($id, $written, $permissions){
		//Sets a cookie storing the user's id and authentication hash
		//$id should be the numeric id, not the hr_id
		
		//Check to see if headers have already been sent
		if(headers_sent()){
			trigger_error("Could not set login cookie for $id: headers already sent.", E_USER_NOTICE);
			return false;
		}
		$hash = md5($id.$written.POOL_SALT);
		$cookie_string = implode('|||', array($id, $hash));
		if(!setcookie('poolauth', $cookie_string, time()+31536000, '/')) trigger_error('Failed setting poolauth', E_USER_WARNING);
		if(!setcookie('poolid', $id, time()+31536000, '/')) trigger_error('Failed setting poolid', E_USER_WARNING);
		if(!setcookie('poollevel', $permissions, time()+31536000, '/')) trigger_error('Failed setting poollevel', E_USER_WARNING);
		return true;
	}
	
	public static function clear_cookie($id=0){
		//Clears the user's authentication cookie
		
		//Check to see if headers have already been sent
		if(headers_sent()){
			trigger_error("Could not clear login cookie: headers already sent.", E_USER_NOTICE);
			return false;
		}
		setcookie('poolauth', false, time() - 31536000);
		setcookie('poolid', false, time() - 31536000);
		setcookie('poollevel', false, time() - 31536000);
		return true;		
	}
	
	public static function cookie_auth(){
		//Does an authentication based on the hash stored in a cookie
		//If authentication fails, return false
		//If authentication succeeds, return the permissions level of the person	
		
		//If no cookie is set, it's hard to authenticate...
		if(!isset($_COOKIE['poolauth'])) return false;
		
		//Get the database
		$data = &$GLOBALS['data'];
		
		//Split the cookie string into id and hash sections
		$cookie_arr = explode('|||', $_COOKIE['poolauth']);
		$id = $cookie_arr[0];
		$hash = $cookie_arr[1];
		
		//Get user info from db
		$q = "SELECT `id`, `permissions`, `written`, `hr_id`
				FROM `person` 
				WHERE `person`.`id` = ?";
		$result = $data->getRow($q, array($id));
		if(DB::isError($result)){
			trigger_error("Could not authenticate id '$id' using cookie.", E_USER_WARNING);
			return false;
		} else {
			//Recalculate the hash from db data and compare
			$db_hash = md5($result['id'].$result['written'].POOL_SALT);
			if($db_hash != $hash) return false;
			self::update_time($id);
			return $result;
		}		
	}
	
	public static function update_time($id){
		//Updates last_login for the passed id to now
		//If $id is an int, it is `person`.`id`
		//If $id is a string, it is `person`.`hr_id`
		//Returns false on error, true otherwise
		
		//Get the database
		$data = &$GLOBALS['data'];
		
		//Check whether this is an id or an hr_id
		$col = is_numeric($id) ? 'id' : 'hr_id';
		$q = "UPDATE `person` SET `last_login` = NOW() WHERE `$col` = ?";
		$result = $data->getAll($q, array($id));
		if(DB::isError($result)){
			trigger_error("Could not update time for user id '$id' in 'person' table.", E_USER_WARNING);
			return false;
		} else return true;			
	}
	
	public static function name_to_hr_id($name){
		//Creates a new hr_id from a written name.
				
		//Get the database
		$data = &$GLOBALS['data'];
		
		//Build the text part of the name
		$name_split = explode(' ', $name);
		$fname = strtolower(array_shift($name_split));
		$lname = strtolower(implode('', $name_split));
		$pool_name = substr($lname.$fname, 0, 30);
		
		$escaped = $data->escapeSimple($pool_name).'%';
		
		//Find out if this name already exists in the db
		$q = "SELECT COUNT(*) as count FROM `person` WHERE `hr_id` LIKE 'per_$escaped'";
		$result = (int) $data->getOne($q, array());
		$result++;
		$pool_name .= sprintf("%02d", $result);
		return $pool_name;
	}
	
	public static function resolve_hr($hr_id){
	//Turns an hr_id into a numeric one
	
		$data = &$GLOBALS['data'];
		$q = "SELECT `id` FROM `person` WHERE `hr_id` = ?";
		$result = $data->getOne($q, array($hr_id));
		if(DB::isError($result)){
			trigger_error("Could not resolve hr_id '$hr_id' in 'person' table.", E_USER_WARNING);
			return false;
		} else {
			return $result;
		}	
	}
	
	public static function get_all_written(){
	//Returns an array with id => [hr_id: '', written: '']

		$data = &$GLOBALS['data'];
		$q = 'SELECT `id`, `hr_id`, `written` FROM `person` ORDER BY `written`';
		$result = $data->getAll($q, array());
		if(DB::isError($result)){
			trigger_error("Error retrieving data from person table in person->get_all_written.", E_USER_WARNING);
			return false;
		} else {
			return $result;
		}
	}
	
	public static function get_written($person_id){
	//Returns a single written name for the passed person id
		
		$data = &$GLOBALS['data'];
		$q = 'SELECT `written` FROM `person` WHERE `id` = ?';
		$result = $data->getOne($q, array($person_id));
		if(DB::isError($result)){
			trigger_error("Error retrieving data from person table in person->get_written.", E_USER_WARNING);
			return false;
		} else {
			return $result;
		}
	}
	
	public static function get_hr_id($person_id){
	//Returns a single hr_idfor the passed person id
		
		$data = &$GLOBALS['data'];
		$q = 'SELECT `hr_id` FROM `person` WHERE `id` = ?';
		$result = $data->getOne($q, array($person_id));
		if(DB::isError($result)){
			trigger_error("Error retrieving data from person table in person->get_hr_id.", E_USER_WARNING);
			return false;
		} else {
			return $result;
		}	
	}
	
	public static function get_creds($person_id){
	//returns all credibilities for the passed person id in person_ratings in an assoc array
		$data = &$GLOBALS['data'];
		
		$q = 'SELECT `category`, `rating`, `basis` FROM `person_ratings` WHERE `person_id` = ?';
		$result = $data->getAll($q, array($person_id));
		if(DB::isError($result)){
			trigger_error("Error retrieving data from person table in person->get_written.", E_USER_WARNING);
			return false;
		} else {
			$assoc = array();
			foreach($result as $cred) $assoc[$cred['category']] = array('rating' => $cred['rating'], 'basis' => $cred['basis']);
			return $assoc;
		}		
	}
	
	public static function get_cached_js($person_id){
	//writes the js_cache for the passed person ID to stdout
		$data = &$GLOBALS['data'];
		
		$q = 'SELECT `js_cache` FROM `person` WHERE `id` = ?';
		$result = $data->getOne($q, array($person_id));
		if(DB::isError($result)){
			trigger_error("Error retrieving js cache from person table in person->get_cached_js.", E_USER_WARNING);
			return false;
		} else {
			echo $result;
		}
	}
	
	public static function cookie_admin_check(){
	//Checks to see if the person currently cookied is an admin (also makes sure their auth string matches what's in the database)
		$data = &$GLOBALS['data'];
		
		$q = 'SELECT `written`, `permissions` FROM `person` WHERE `id` = ?';
		$result = $data->getRow($q, array($_COOKIE['poolid']));
		if(DB::isError($result)){
			trigger_error("Error in Pool_Person->cookie_admin_check().", E_USER_WARNING);
			return false;		
		}
		if(count($result) == 0) return false;
		
		//Split the cookie string into id and hash sections
		$cookie_arr = explode('|||', $_COOKIE['poolauth']);
		$id = $cookie_arr[0];
		$hash = $cookie_arr[1];
		
		//Recalculate the hash from db data and compare
		$db_hash = md5($_COOKIE['poolid'].$result['written'].POOL_SALT);
		if($db_hash != $hash) return false;
		if($result['permissions'] == 2) return true;
		else return false;
	}
	
	public static function password_change($id, $old_pass, $new_pass, $is_admin){
	//Changes a user's password.  If $is_admin is true, don't check the old password
		$data = &$GLOBALS['data'];
		
		if(!$is_admin){
			$q = 'SELECT `id` FROM `person` WHERE `id` = ? AND `password` = PASSWORD(?)';
			$result = $data->getOne($q, array($id, $old_pass));
			if(DB::isError($result)){
				trigger_error("Error in Pool_Person->password_change().", E_USER_WARNING);
				return false;		
			}
			if(!$result) return false;
		}
		$q = 'UPDATE `person` SET `password` = PASSWORD(?) WHERE `id` = ?';
		$result = $data->query($q, array($new_pass, $id));
		if(DB::isError($result)){
			trigger_error("Error in Pool_Person->password_change().", E_USER_WARNING);
			return false;		
		}
		return true;
	}
	
	public static function set_self_rating($id, $category, $rating){
	//Sets a person's self rating for a given category
		$data = &$GLOBALS['data'];
		
		$hr_id = Pool_Person::get_hr_id($id);
		$q = 'INSERT INTO `person_ratings` (`person_id`, `category`, `hr_id`, `self_rating`) VALUES (?,?,?,?)
				ON DUPLICATE KEY UPDATE `self_rating` = VALUES(self_rating)';
		$result = $data->query($q, array($id, $category, $category.$hr_id, $rating));
		if(DB::isError($result)){
			trigger_error("Error in Pool_Person->set_self_rating().", E_USER_WARNING);
			return false;		
		}		
		return true;
	}
	
	public static function set_field($id, $field, $value){
	//Sets a field in the person table
		$data = &$GLOBALS['data'];
		
		$q = "UPDATE `person` SET `$field` = ? WHERE `id` = ?";
		$result = $data->query($q, array($value, $id));
		if(DB::isError($result)){
			trigger_error("Error in Pool_Person->set_field($field).", E_USER_WARNING);
			return false;		
		}		
		return true;		
	}
	
	public static function get_recent($person_id){
	//Gets a list of the most recently modified projects this person has contributed to
		$data = &$GLOBALS['data'];
		
		if(!is_numeric($person_id)) $person_id = Pool_Person::resolve_hr($person_id);
		
		$q = 'SELECT `project`.`description` AS description, `project`.`title` AS title, `project`.`id` AS id, `pool_type` AS pool_type,
					`project`.`thumb_id` AS thumb_id , `project`.`rating` AS rating, `project`.`last_modified` AS date
				FROM `project` , `project_artifact` , `person_artifact`
				WHERE `person_artifact`.`person_id` =?
				AND `person_artifact`.`artifact_type` = `project_artifact`.`artifact_type`
				AND `project_artifact`.`artifact_id` = `person_artifact`.`artifact_id`
				AND `project`.`id` = `project_artifact`.`project_id`
				GROUP BY `project`.`id`
				ORDER BY `project`.`last_modified` DESC LIMIT 0,5';
		$result = $data->getAll($q, array($person_id));
		if(DB::isError($result)){
			trigger_error("Error in Pool_Person->get_recent().", E_USER_WARNING);
			return false;		
		}		
		return $result;	
	}
	
	public static function get_recent_reviews($person_id){
	//Gets a list of the most recently reviewed projects this person has contributed to
		$data = &$GLOBALS['data'];
		
		if(!is_numeric($person_id)) $person_id = Pool_Person::resolve_hr($person_id);
		
		$q = 'SELECT `project`.`title` AS title, `project`.`id` AS id, `project`.`pool_type` AS pool_type,
					`project`.`thumb_id` AS thumb_id , `review`.`description` AS description, `review`.`rating` AS rating,
					`review`.`date` AS date, `review`.`recorder_id` AS `recorder`, `person`.`written` AS written
				FROM `project` , `project_artifact` , `person_artifact`, `review`, `person`
				WHERE `person_artifact`.`person_id` =?
					AND `person_artifact`.`artifact_type` = `project_artifact`.`artifact_type`
					AND `project_artifact`.`artifact_id` = `person_artifact`.`artifact_id`
					AND `project`.`id` = `project_artifact`.`project_id`
					AND `review`.`artifact_id` = `person_artifact`.`artifact_id`
					AND `review`.`artifact_type` = `person_artifact`.`artifact_type`
					AND `review`.`recorder_id` = `person`.`id`
				GROUP BY `review`.`id`
				ORDER BY `review`.`date` DESC LIMIT 0,15';
		$result = $data->getAll($q, array($person_id));
		if(DB::isError($result)){
			trigger_error("Error in Pool_Person->get_recent_reviews().", E_USER_WARNING);
			return false;		
		}		
		return $result;	
	}
	
	public static function get_recent_reviews_by($person_id){
	//Gets a list of the most recently reviews this person has contributed
		$data = &$GLOBALS['data'];
		
		if(!is_numeric($person_id)) $person_id = Pool_Person::resolve_hr($person_id);
		
		$q = 'SELECT `project`.`title` AS title, `project`.`id` AS id, `project`.`pool_type` AS pool_type,
					`project`.`thumb_id` AS thumb_id , `review`.`description` AS description, `review`.`rating` AS rating,
					`review`.`date` AS date, `review`.`recorder_id` AS `recorder`, `person`.`written` AS written
				FROM `project` , `project_artifact` , `person_artifact`, `review`, `person`
				WHERE `review`.`recorder_id` =?
					AND `review`.`artifact_id` = `person_artifact`.`artifact_id`
					AND `review`.`artifact_type` = `person_artifact`.`artifact_type`
					AND `review`.`recorder_id` = `person`.`id`
					AND `person_artifact`.`artifact_type` = `project_artifact`.`artifact_type`
					AND `project_artifact`.`artifact_id` = `person_artifact`.`artifact_id`
					AND `project`.`id` = `project_artifact`.`project_id`
				GROUP BY `review`.`id`
				ORDER BY `review`.`date` DESC LIMIT 0,15';
		$result = $data->getAll($q, array($person_id));
		if(DB::isError($result)){
			trigger_error("Error in Pool_Person->get_recent_reviews().", E_USER_WARNING);
			return false;		
		}		
		return $result;	
	}
}
?>