<?PHP
//Abstract Pool Project Class .1
//This implements a generic pool project class that should be extended for each pool subtype

require_once('db_init.php');

abstract class Pool{

	//public functions
	public function __construct(){
		$this->data = &$GLOBALS['data'];
		$this->read_ini(POOL_INI_PATH.$this->ini_file);
	}
	
	public function write_pool_array($limit = 200){
		//write out the javascript array for the most recent $limit projects
		//Get the projects in question
		if($limit == 'all') $q = "SELECT `js_cache_short` FROM `project` WHERE `pool_type` = '$this->pool_type' ORDER BY `last_modified` DESC";	else $q = "SELECT `js_cache_short` FROM `project` WHERE `pool_type` = '$this->pool_type' ORDER BY `last_modified` DESC LIMIT 0,$limit";
		$result = $this->data->getAll($q, array());
		if(!$this->check_db($result, 'write_pool_array')) return false;
		//write out array
		require_once('smarty_init.php');
		$smarty = &$GLOBALS['smarty'];
		
		$smarty->assign('fishes', $result);
		$smarty->left_delimiter = '<{';
		$smarty->right_delimiter = '}>';		
		$smarty->display('js_pool_array_short.tpl');
		$smarty->left_delimiter = '{';
		$smarty->right_delimiter = '}';
	}
	
	public function get_recent(){
		//Gets the recently modified projects for this pool
		$q = 'SELECT `project`.`description` AS description, `project`.`title` AS title, `project`.`id` AS id, `pool_type` AS pool_type,
					`project`.`thumb_id` AS thumb_id , `project`.`rating` AS rating, `project`.`last_modified` AS date
				FROM `project`
				WHERE `project`.`pool_type` = ?
				GROUP BY `project`.`id`
				ORDER BY `project`.`last_modified` DESC LIMIT 0,5';
		$result = $this->data->getAll($q, array($this->pool_type));
		if(DB::isError($result)){
			trigger_error("Error in Pool->get_recent().", E_USER_WARNING);
			return false;	
		}		
		return $result;	
	}
	
	//protected functions
	protected function check_db($result, $func){
		//Return true if $result is not DB::Error, false and an error otherwise
		if(DB::isError($result)){
			trigger_error("Pool->$func failed: $this->data->last_query \n".$result->getMessage(), E_USER_WARNING);	
			return false;
		} else return true;
	}
	
	protected function read_ini($filename){
		//Reads a pool ini file and sets variables appropriately
		//See sample.ini for details on ini structure
		$init_array = parse_ini_file($filename, true);
		$this->pool_type = $init_array['general']['type'];
		$this->has_artifacts = $init_array['general']['has_artifacts'] == 'true' ? true : false;
		if($this->has_artifacts){
			//Determine what stages are in this pool type and set container objects for them
			$this->stages = explode(',', $init_array['general']['stages']);
			foreach($this->stages as $stage) $this->$stage = array();
			
			//Determine what aspects each stage has
			foreach($init_array['aspects'] as $stage=>$aspects){
				$stage_name = $stage.'_aspects';
				$exploded = explode(',', $aspects);
				$this->$stage_name = $exploded;
			}
			
			//Determine what ratings each stage is subject to
			foreach($init_array['ratings'] as $stage=>$ratings){
				$stage_name = $stage.'_ratings';
				$exploded = explode(',', $ratings);
				$this->$stage_name = $exploded;	
			}
		}
	}	
}

abstract class Ref_Pool extends Pool{
	public function write_pool_array($limit = 200){
		//write out the javascript array for the most recent $limit projects
		//Get the projects in question
		if($limit == 'all') $q = "SELECT `js_cache_short` FROM `reference` WHERE `pool_type` = '$this->pool_type' ORDER BY `last_modified` DESC";	else $q = "SELECT `js_cache_short` FROM `reference` WHERE `pool_type` = '$this->pool_type' ORDER BY `last_modified` DESC LIMIT 0,$limit";
		$result = $this->data->getAll($q, array());
		if(!$this->check_db($result, 'write_pool_array')) return false;
		//write out array
		require_once('smarty_init.php');
		$smarty = &$GLOBALS['smarty'];
		
		$smarty->assign('fishes', $result);
		$smarty->left_delimiter = '<{';
		$smarty->right_delimiter = '}>';		
		$smarty->display('js_pool_array_short.tpl');
		$smarty->left_delimiter = '{';
		$smarty->right_delimiter = '}';
	}
	
	public function get_recent(){
		//Gets the recently modified projects for this pool
		$q = 'SELECT `reference`.`description` AS description, `reference`.`title` AS title, `reference`.`id` AS id, `pool_type` AS pool_type,
					`reference`.`thumb_id` AS thumb_id , `reference`.`rating` AS rating, `reference`.`last_modified` AS date
				FROM `reference`
				WHERE `reference`.`pool_type` = ?
				GROUP BY `reference`.`id`
				ORDER BY `reference`.`last_modified` DESC LIMIT 0,5';
		$result = $this->data->getAll($q, array($this->pool_type));
		if(DB::isError($result)){
			trigger_error("Error in Pool->get_recent().", E_USER_WARNING);
			return false;	
		}		
		return $result;	
	}
}

class Art_Pool extends Pool{
	public $ini_file = 'art_pool.ini';
}

class Art_Ref_Pool extends Ref_Pool{
	public $ini_file = 'art_ref_pool.ini';	
}

class Code_Pool extends Pool{
	public $ini_file = 'code_pool.ini';	
}
class Code_Ref_Pool extends Ref_Pool{
	public $ini_file = 'code_ref_pool.ini';	
}

?>