<?PHP
//Intent Class .1
//This implements a class for intents of all pool projects
//Requires the db_init, Person.class and error_init files

require_once('db_init.php');
require_once('person.class.php');
require_once('pool_project.class.php');
require_once('review.class.php');

class Pool_Intent{
	//Public variables (from db)
	public $id = false;				//ID of the intent
	public $hr_id = '';				//Human readable ID of the intent
	public $pool_type = '';			//Project type
	public $title = '';				//Title of the intent/project
	public $date = '';				//Date this intent was last updated
	public $description = '';		//Full text description of the intent
	public $uri_view = '';			//URI to view more on this intent
	public $requests = '';			//Requests for help on this intent
	public $recorder_id = '';		//ID of the recorder of this intent
	public $js_cache = '';			//Cached JS object for this intent
	
	public $project_id = '';		//ID of the project this intent is a part of
	//Array of contributors for this intent
	//This will look like $contributors['contributor_id'] = array('id' => 39, 'written' = 'Contrib User', 'roles' => 'conceptual,technical');
	public $contributors = array();
	//Array of reviews for this intent
	//This is an array of Review objects
	public $reviews = array();
	//Array of aspects for this intent
	//This will look like $aspects['aspect_name'] = aspect_value
	public $aspects = array();
	//Array of ratings for this intent
	//This will look like $rating['category'] = array('rating' => 6.55, 'basis' => 5);
	public $ratings = array();

	//Protected db object
	//This is filled in the constructor
	protected $data = '';
					
	//Abstract functions				

	//Generic functions
	public function __construct($construct_var){
		//Generic constructor
		//$construct_var can be either a string or an array
		//If $construct_var is a string, it is assumed that you are loading a review from the db
		//If $construct_var is an array, it is assumed you are adding a new review
		
		//Set up the db object
		$this->data = &$GLOBALS['data'];
		
		if(is_array($construct_var)) $this->new_intent($construct_var);
		else{
			$this->id = $construct_var;
			$this->load_intent();
		}
	}
	
	public function new_intent($form_data){
		//saves a new intent to the database and creates a project to contain it
		//if $form_data['intent_id'] != -1, it updates the old intent
		//print_r($form_data);
		
		//Create the hr_id
		if($form_data['intent_id'] == -1) $form_data['hr_id'] = $this->new_hr_id($form_data['pool_type'], $form_data['title']);
		else $form_data['hr_id'] = Pool_Project::id_to_hr($form_data['intent_id'], 'intent');
		$this->hr_id = $form_data['hr_id'];
		//First create a project
		$class_name = ucfirst($form_data['pool_type']).'_Project';
		$oProject = new $class_name($form_data);
		
		//Insert the intent table data
		$q = 'INSERT INTO `intent` (`hr_id`, `pool_type`, `title`, `date`, `description`, `uri_view`, `requests`, `recorder_id`)
				VALUES (?,?,?,NOW(),?,?,?,?) 
				ON DUPLICATE KEY UPDATE `title` = VALUES(`title`),
										`description` = VALUES(`description`), 
										`date` = VALUES(`date`),
										`uri_view` = VALUES(`uri_view`),
										`requests` = VALUES(`requests`)';
		$intent_data = array($form_data['hr_id'], $form_data['pool_type'], $form_data['title'], $form_data['description'], $form_data['uri_view'], $form_data['requests'], $form_data['recorder_id']);
		$result = $this->data->query($q, $intent_data);
		if(!$this->check_db($result, 'new_intent')) return false;
		
		//Get back the id you just created
		$q = 'SELECT `id` FROM `intent` WHERE `hr_id` = ?';
		$this->id = $this->data->getOne($q, array($form_data['hr_id']));
		
		//Connect contributors
		$q = 'DELETE FROM `person_artifact` WHERE `artifact_id` = ? AND `artifact_type` = ?';
		$result = $this->data->query($q, array($this->id, 'intent'));
		if(!$this->check_db($result, 'new_intent')) return false;
		
		$q = 'INSERT INTO `person_artifact` (`person_id`, `artifact_id`, `artifact_type`, `role`, `hr_link`)
				VALUES (?, ?, ?, ?, ?)
				ON DUPLICATE KEY UPDATE `hr_link` = `hr_link`';
		foreach($form_data['contributors'] as $person_id => $roles){
			$oPerson = new Pool_Person($person_id);
			foreach($roles as $role){
				$person_array = array($person_id, $this->id, 'intent', $role, $oPerson->hr_id.'_'.$role);
				$result = $this->data->query($q, $person_array);
				if(!$this->check_db($result, 'new_intent')) return false;
			}
		}
		
		//Connect aspects
		$q = 'DELETE FROM `aspects` WHERE `artifact_id` = ? AND `artifact_type` = ?';
		$result = $this->data->query($q, array($this->id, 'intent'));
		if(!$this->check_db($result, 'new_intent')) return false;		
		
		$q = 'INSERT INTO `aspects` (`artifact_id`, `artifact_type`, `aspect`, `hr_link`, `value`)
				VALUES (?, ?, ?, ?, ?) ON DUPLICATE KEY UPDATE `value` = VALUES(`value`)';
		foreach($form_data['aspects'] as $aspect){
			if($aspect != ''){
				$aspect_array = array($this->id, 'intent', $aspect, $form_data['hr_id'].'_'.$aspect, 1);
				$result = $this->data->query($q, $aspect_array);
				if(!$this->check_db($result, 'new_intent')) return false;
			}
		}
		
		//Connect project
		$q = 'INSERT INTO `project_artifact` (`project_id`, `artifact_id`, `artifact_type`, `hr_link`)
				VALUES (?, ?, ?, ?) ON DUPLICATE KEY UPDATE `hr_link` = `hr_link`';
		$result = $this->data->query($q, array($oProject->id, $this->id, 'intent', $oProject->hr_id.'_'.$this->hr_id));
		if(!$this->check_db($result, 'new_intent')) return false;		
		
		//Update javascript caches
		$this->load_intent();
		$this->save_js();
		$oProject->reload_all();
		$oProject->save_js();
		$oProject->save_js_short();
	}
	
	public function load_intent(){
		//Loads data from the intent table
		
		//make sure the id has been set
		if(!$this->check_id('load_review')) return false;
		
		//Get the intent table data
		$q = 'SELECT * FROM `intent` WHERE `id` = ?';
		$result = $this->data->getRow($q, array($this->id));
		if(!$this->check_db($result, 'load_intent')) return false;
		foreach($result as $key=>$col) $this->$key = $col;
		
		//Fill in the rest of the data from other tables
		$this->contributors = Pool_Project::get_contributor_list('intent', $this->id);
		$this->project_id = Pool_Project::resolve_artifact_id('intent', $this->id);
		$this->reviews = Pool_Project::get_reviews('intent', $this->id);
		$this->aspects = Pool_Project::get_aspects('intent', $this->id);
		$this->ratings = Pool_Project::get_ratings('intent', $this->id);
	}
	
	public function write_js(){
		//writes the JS array for this intent
		
		require_once('smarty_init.php');
		$smarty = &$GLOBALS['smarty'];
		
		$smarty->assign('intent', $this);
		$smarty->left_delimiter = '<{';
		$smarty->right_delimiter = '}>';
		$smarty->display('js_intent_array.tpl');
		$smarty->left_delimiter = '{';
		$smarty->right_delimiter = '}';
	}
	
	public function save_js(){
		//writes the JS array for this review
		
		require_once('smarty_init.php');
		$smarty = &$GLOBALS['smarty'];
		
		$smarty->assign('intent', $this);
		$smarty->left_delimiter = '<{';
		$smarty->right_delimiter = '}>';
		$js_cache = $smarty->fetch('js_intent_array.tpl');
		$smarty->left_delimiter = '{';
		$smarty->right_delimiter = '}';
		
		$q = 'UPDATE `intent` SET `js_cache` = ? WHERE `id` = ?';
		$this->data->query($q, array($js_cache, $this->id));
	}	
	
	//Protected functions
	protected function check_id($func){
		//Return true if the id has been set, false and an error otherwise
		if($this->id == false){
			trigger_error("Pool_Intent->$func failed: No id defined", E_USER_WARNING);
			return false;
		} else return true;
	}

	protected function check_db($result, $func){
		//Return true if $result is not DB::Error, false and an error otherwise
		if(DB::isError($result)){
			trigger_error("Pool_Intent->$func failed: $this->data->last_query \n".$result->getMessage(), E_USER_WARNING);	
			return false;
		} else return true;
	}
	
	//static methods
	public static function new_hr_id($artifact_type, $title){
		//Creates a new hr_id based on the title of an intent
		$data = &$GLOBALS['data'];
	
		$title = str_replace(' ', '', $title);
		if(strlen($title) > 18) $title = substr($title, 0, 17);
		if(strlen($artifact_type) > 3) $artifact_type = substr($artifact_type, 0, 2);
		$hr_name = $artifact_type.'_'.$title;
		$q = 'SELECT count(*) FROM `intent` WHERE `hr_id` LIKE ?';
		$result = $data->getOne($q, array($hr_name.'%'));
		if(!self::check_db($result, 'new_hr_id')) return false;
		$result++;
		$hr_num = sprintf("%02d", $result);
		return $hr_name.$hr_num;
	}
}
?>