<?PHP
//Reference Class .1
//This implements a class for references of all pool types
//Requires the db_init, Person.class and error_init files

require_once('db_init.php');
require_once('review.class.php');

class Pool_Reference{
	//Public variables (from db)
	public $id = false;				//ID of the reference
	public $hr_id = '';				//Human readable ID of the reference
	public $pool_type = '';			//Reference type
	public $title = '';				//Title of the reference
	public $date_published = '';	//Date this reference was published
	public $citation = '';			//Citation for the work
	public $description = '';		//Full text description of the reference
	public $source_url = '';		//URI to view more on this reference
	public $recorder_id = '';		//ID of the recorder of this reference
	public $js_cache = '';			//Cached JS object for this reference
	public $js_cache_short = '';	//Short version of the cached JS object
	public $last_modified = '';		//Last modified on
	
	//Array of attributions for this reference
	//This is an array of Attribution objects
	public $attributions = array();
	//Array of reviews for this reference
	//This is an array of Review objects
	public $reviews = array();

	//This will look like $aspects['aspect_name'] = aspect_value
	public $aspects = array();

	//Array of ratings for this reference
	//This will look like $rating['category'] = array('rating' => 6.55, 'basis' => 5);
	public $ratings = array();
	//Array of relationships for this reference
	//This will look like 	relationship: 'usesInCombination',
	//						subject: '384',
	//						subject_type: 'art',
	//						subject_title: 'Feral Festival II',
	//						predicate: '1',
	//						predicate_type: 'artreference',
	//						predicate_title: 'Feral Animals',
	//						description: 'The feral festival requires the use of feral animals.',
	//						recorder_id: '4',
	//						recorder_written: 'John Bell'
	public $relationships = array();


	//Protected db object
	//This is filled in the constructor
	protected $data = '';
					
	//Abstract functions				

	//Generic functions
	public function __construct($construct_var){
		//Generic constructor
		//$construct_var can be either a string or an array
		//If $construct_var is a string, it is assumed that you are loading a reference from the db
		//If $construct_var is an array, it is assumed you are adding a new reference
		
		//Set up the db object
		$this->data = &$GLOBALS['data'];
		
		if(is_array($construct_var)) $this->new_reference($construct_var);
		else{
			$this->id = $construct_var;
			$this->load_reference();
		}
	}
	
	
	
	public function new_reference($form_data){
		//saves a new reference to the database
		//if $form_data['reference_id'] != -1, it updates the old reference
		//print_r($form_data);
		
		//Create the hr_id
		if($form_data['reference_id'] == -1) $form_data['hr_id'] = $this->new_hr_id($form_data['pool_type'], $form_data['title']);
		else {
			$form_data['hr_id'] = Pool_Reference::id_to_hr($form_data['reference_id']);	
		}
		$this->hr_id = $form_data['hr_id'];

		//Insert the reference table data
		$q = 'INSERT INTO `reference` (`title`, `hr_id`, `pool_type`, `date_published`, `citation`, `source_url`, `description`, `recorder_id`, `last_modified`) 
				VALUES(?,?,?,?,?,?,?,?,NOW()) 
				ON DUPLICATE KEY UPDATE `title` = VALUES(`title`),
										`description` = VALUES(`description`), 
										`citation` = VALUES(`citation`),
										`date_published` = VALUES(`date_published`),
										`source_url` = VALUES(`source_url`),
										`recorder_id` = VALUES(`recorder_id`),
										`last_modified` = NOW()';
		$reference_data = array($form_data['title'], $form_data['hr_id'], $form_data['pool_type'], $form_data['date_published'], $form_data['citation'], $form_data['source_url'], $form_data['description'], $form_data['recorder_id']);
		$result = $this->data->query($q, $reference_data);
		if(!$this->check_db($result, 'new_reference')) return false;
	
		//Get back the id you just created
		$q = 'SELECT `id` FROM `reference` WHERE `hr_id` = ?';
		$this->id = $this->data->getOne($q, array($form_data['hr_id']));

		//Connect contributors
		$q = 'DELETE FROM `reference_attribution` WHERE `reference_id` = ?';
		$result = $this->data->query($q, array($this->id));
		if(!$this->check_db($result, 'new_reference')) return false;
		
		$oAttribution = new Pool_Attribution($form_data['attribution_id']);
		
		$q = 'INSERT INTO `reference_attribution` (`reference_id`, `attribution_id`, `hr_link`)
				VALUES (?, ?, ?)
				ON DUPLICATE KEY UPDATE `hr_link` = `hr_link`';
		$result = $this->data->query($q, array($this->id, $oAttribution->id, $form_data['title'].'_'.$oAttribution->name));
		if(!$this->check_db($result, 'new_reference')) return false;

		//Connect subjects
		$q = "DELETE FROM `reference_subject` WHERE `reference_id` = ?";
		$result = $this->data->query($q, array($this->id));
		foreach($form_data['subjects'] as $subject){
			$subject_hr_id = $this->data->getOne('SELECT `hr_id` FROM `subject` WHERE `id` = ?', array($subject));
			$q = 'INSERT INTO `reference_subject` (`reference_id`, `subject_id`, `hr_link`) VALUES (?, ?, ?)
					ON DUPLICATE KEY UPDATE `reference_id` = `reference_id`';
			$result = $this->data->query($q, array($this->id, $subject, $this->hr_id . '_' . $subject_hr_id));
			if(!$this->check_db($result, 'new_reference')) return false;
		}

		//Connect aspects
		$q = 'DELETE FROM `aspects` WHERE `artifact_id` = ? AND `artifact_type` = ?';
		$result = $this->data->query($q, array($this->id, 'reference'));
		if(!$this->check_db($result, 'new_reference')) return false;		
		
		$q = 'INSERT INTO `aspects` (`artifact_id`, `artifact_type`, `aspect`, `hr_link`, `value`)
				VALUES (?, ?, ?, ?, ?) ON DUPLICATE KEY UPDATE `value` = VALUES(`value`)';
		foreach($form_data['aspects'] as $aspect){
			if($aspect != ''){
				$aspect_array = array($this->id, 'reference', $aspect, $this->hr_id.'_'.$aspect, 1);
				$result = $this->data->query($q, $aspect_array);
				if(!$this->check_db($result, 'new_reference')) return false;
			}
		}
		//Add custom rights, if they exist
		$aspect_array = array($this->id, 'reference', 'custom-rights', $form_data['hr_id'].'_custom_rights', $form_data['custom_rights']);
		$result = $this->data->query($q, $aspect_array);
		if(!$this->check_db($result, 'new_reference')) return false;
		
		$temp = new Pool_Reference($this->id);
		$temp->load_relations();
		$temp->load_reviews();
		$temp->set_rating_and_basis();
		$temp->load_ratings();
		//Update javascript caches
		$temp->save_js_short();
		$temp->save_js();
		return $temp;
	}
		
	public function load_reference(){
		//Loads data from the reference table
		
		//make sure the id has been set
		if(!$this->check_id('load_reference')) return false;
		
		//Get the reference table data
		$q = 'SELECT *, UNIX_TIMESTAMP(`last_modified`) AS last_modified_unix FROM `reference` WHERE `id` = ?';
		$result = $this->data->getRow($q, array($this->id));
		if(!$this->check_db($result, 'load_reference')) return false;
		foreach($result as $key=>$col) $this->$key = $col;
		
		//Fill in the rest of the data from other tables
		$this->attribution = $this->data->getOne('SELECT `attribution_id` FROM `reference_attribution` WHERE `reference_id` = ?', array($this->id));
		$this->attribution_written = $this->data->getOne('SELECT `name` FROM `attribution` WHERE `id` = ?', array($this->attribution));
		$this->recorder_written = $this->data->getOne('SELECT `written` FROM `person` WHERE `id` = ?', array($this->recorder_id));
		$result = $this->data->getAll('SELECT `reference_subject`.`subject_id`, `subject`.`written` FROM `reference_subject`, `subject` WHERE `reference_subject`.`reference_id` = ? AND `subject`.`id` = `reference_subject`.`subject_id`', array($this->id));
		if(!$this->check_db($result, 'load_reference')) return false;
		$this->subjects = array();
		foreach($result as $subject) $this->subjects[$subject['subject_id']] = $subject['written'];

		$this->aspects = Pool_Project::get_aspects('reference', $this->id);

		$this->load_relations();
		$this->load_reviews();
		$this->load_ratings();
	}

	public function reload_all(){
	//Reloads this object from the database
		$this->subratings=array();
		$this->reviews=array();
		$this->subjects=array();
		$this->relationships = array();	
		$this->load_reference();
	}

	public function set_rating_and_basis(){
		//Sets the rating and basis for this reference
		//Assumes that load_contributors and load_reviews have already been run.
		
		//First get the credibilities of everybody who has reviewed the project
		$creds = $this->get_reviewer_creds();
		
		//Apply credibilities to review ratings
		//$ratings is an array of rating categories.  Each category has a weight and a total.  Weight is the sum of credibilities, while total is the sum of (credibilities * ratings)
		$ratings = array();
		foreach($this->reviews as $review){
			foreach($review->subratings as $category => $subrating){
				//Check to see if this person has a credibility for this category of rating.  If it does not exist, set it to 1
				if(!isset($creds[$review->recorder_id][$category])) $creds[$review->recorder_id][$category] = 1;
				//If this is the first rating of the category, initialize it in the ratings array
				if(!isset($ratings[$category])) $ratings[$category] = array('weight' => 0, 'total' => 0, 'basis' => 0);
				//Add the rating values
				$ratings[$category]['weight'] += $creds[$review->recorder_id][$category] > 1 ? $creds[$review->recorder_id][$category] : 1;
				$ratings[$category]['total'] += $creds[$review->recorder_id][$category] * $subrating['rating'];
				$ratings[$category]['basis']++;
			}
		}
		
		//Calculate the weighted ratings, as well as the overall rating and basis for the project
		$overall = $num_ratings = 0;
		foreach($ratings as $category => $subrating){
			if($subrating['weight'] == 0) $subrating['final'] = $subrating['total'];
			else $subrating['final'] = $subrating['total'] / $subrating['weight'];
			$overall += $subrating['final'];
			$ratings[$category] = $subrating;
			$num_ratings++;
		}
		if($num_ratings > 0) $overall = $overall / $num_ratings;
		else $overall = 0;
		$basis = count($this->reviews);

		//Store the new rating, subratings, and basis
		$q = 'UPDATE `reference` SET `rating` = ?, `basis` = ? WHERE `id` = ?';
		$result = $this->data->query($q, array($overall, $basis, $this->id));
		if(!$this->check_db($result, 'set_rating_and_basis')) return false; 

		foreach($ratings as $category => $subrating){
			$values_arr = array();
			$data_arr = array();
					
			$values_arr[] = '(?, "reference", ?, ?, ?, ?)';
			$data_arr[] = $this->id;
			$data_arr[] = $category;
			$data_arr[] = $subrating['final'];
			$data_arr[] = $subrating['basis'];
			$data_arr[] = $this->hr_id.'_'.$category;
			$values_str = implode(',', $values_arr);
			
			$q = "INSERT INTO `artifact_ratings` (`artifact_id`, `artifact_type`, `category`, `rating`, `basis`, `hr_link`)
					VALUES $values_str ON DUPLICATE KEY UPDATE `rating` = VALUES(`rating`), `basis` = VALUES(`basis`)";
			$result = $this->data->query($q, $data_arr);
			if(!$this->check_db($result, 'set_rating_and_basis')) return false;
		}
		//update the js_cache and js_cache_short for the project
		$this->reload_all();
		$this->save_js();
		$this->save_js_short();
		
		//update the attribution's ratings
		$oContrib = new Pool_Attribution($this->attribution);
		$oContrib->set_rating_and_basis();
				
		return true;
	}


	public function load_relations(){
		//Loads the relationships for this object
		if(!$this->check_id('load_relations')) return false;
		
		$q = 'SELECT * FROM `relations` WHERE (`subject_id` = ? AND `subject_type` = ?) OR (`predicate_id` = ? AND `predicate_type` = ?)';
		$result = $this->data->getAll($q, array($this->id, $this->pool_type, $this->id, $this->pool_type));
		if(!$this->check_db($result, 'load_relations')) return false;
		foreach($result as $relationship) $this->relationships[$relationship['id']] = $relationship;
		
		//Add titles and recorder name
		foreach($this->relationships as &$relationship){
			if(strpos($relationship['subject_type'], 'ref') === false) $q = 'SELECT `title` FROM `project` WHERE `id` = ?';
			else $q = 'SELECT `title` FROM `reference` WHERE `id` = ?';
			$relationship['subject_title'] = $this->data->getOne($q, array($relationship['subject_id']));
			
			if(strpos($relationship['predicate_type'], 'ref') === false) $q = 'SELECT `title` FROM `project` WHERE `id` = ?';
			else $q = 'SELECT `title` FROM `reference` WHERE `id` = ?';
			$relationship['predicate_title'] = $this->data->getOne($q, array($relationship['predicate_id']));			
			
			$q = 'SELECT `written` FROM `person` WHERE `id` = ?';
			$relationship['recorder_written'] = $this->data->getOne($q, array($relationship['recorder_id']));
		}

		return true;
	}

	public function load_reviews(){
		//Loads all reviews for this reference as review objects
		if(!$this->check_id('load_reviews')) return false;
		$q = "SELECT `id` FROM `review` WHERE `artifact_id` = ? AND `artifact_type` = 'reference'";
		$result = $this->data->getAll($q, array($this->id));
		if(!$this->check_db($result, 'load_reviews')) return false;
		foreach($result as $reference_review) $this->reviews[] = new Review($reference_review['id']);
		return true;
	}
	
//	public function load_subratings(){
//		//Loads subratings for this reference
//		if(!$this->check_id('load_subratings')) return false;
//		
//		$q = "SELECT `category`, `rating`, `basis` FROM `artifact_ratings` WHERE `artifact_id` = ? AND `artifact_type` = 'reference'";
//		$result = $this->data->getAll($q, array($this->id));
//		if(!$this->check_db($result, 'load_subratings')) return false;
//		foreach($result as $subrating) $this->subratings[$subrating['category']] = array('rating' => $subrating['rating'], 'basis' => $subrating['basis']);
//		return true;
//	}	

	public function load_ratings(){
		//gets composite ratings for the reference
		$q = 'SELECT `category`, `rating`, `basis` FROM `artifact_ratings` WHERE `artifact_type` = "reference" AND `artifact_id` = ?';
		$result = $this->data->getAll($q, array($this->id));
		if(DB::isError($result)){
			trigger_error("Could not get ratings for reference with id=".$this->id." in Pool_Reference::load_ratings.", E_USER_WARNING);
			return false;
		} else {
			foreach($result as $rating) $this->subratings[$rating['category']] = array('rating' => $rating['rating'], 'basis' => $rating['basis']);
		}			
		
	}
	
	public function get_reviewer_creds(){
		//get the credibilities for everybody who has written reviews of this reference
		
		$q = 'SELECT `person_ratings`.`person_id`, `person_ratings`.`category`, `person_ratings`.`rating`
				FROM `person_ratings`, `review`
				WHERE `review`.`artifact_id` = ?
					AND `review`.`artifact_type` = "reference"
					AND `review`.`recorder_id` = `person_ratings`.`person_id`
					GROUP BY `person_ratings`.`person_id`, `person_ratings`.`category`, `person_ratings`.`rating`';
		$result = $this->data->getAll($q, array($this->id));
		if(!$this->check_db($result, 'get_reviewer_creds')) return false;
		$creds = array();
		foreach($result as $cred){
			if(!isset($creds[$cred['person_id']])) $creds[$cred['person_id']] = array();
			$creds[$cred['person_id']][$cred['category']] = $cred['rating'];
		}
		
		return $creds;
	}
	
	public function write_js(){
		//writes the JS array for this reference
		
		require_once('smarty_init.php');
		$smarty = &$GLOBALS['smarty'];
		
		$smarty->assign('reference', $this);
		$smarty->left_delimiter = '<{';
		$smarty->right_delimiter = '}>';
		$smarty->display('js_reference_array.tpl');
		$smarty->left_delimiter = '{';
		$smarty->right_delimiter = '}';
	}
	
	public function save_js(){
		//writes the JS array for this review
		
		require_once('smarty_init.php');
		$smarty = &$GLOBALS['smarty'];
		
		$smarty->assign('reference', $this);
		$smarty->left_delimiter = '<{';
		$smarty->right_delimiter = '}>';
		$js_cache = $smarty->fetch('js_reference_array.tpl');
		$smarty->left_delimiter = '{';
		$smarty->right_delimiter = '}';
		
		$q = 'UPDATE `reference` SET `js_cache` = ? WHERE `id` = ?';
		$this->data->query($q, array($js_cache, $this->id));
	}	
	
	public function save_js_short(){
		//writes the JS array for this review
		
		require_once('smarty_init.php');
		$smarty = &$GLOBALS['smarty'];
		
		$short_desc = substr($this->description, 0, 400);
		if(strlen($this->description) > 400) $short_desc = $short_desc.'...';
		$smarty->assign('short_desc', $short_desc);
		$smarty->assign('relations', count($this->relationships) + 1);		
		$smarty->assign('reference', $this);
		$smarty->left_delimiter = '<{';
		$smarty->right_delimiter = '}>';
		$js_cache = $smarty->fetch('js_reference_array_short.tpl');
		$smarty->left_delimiter = '{';
		$smarty->right_delimiter = '}';
		
		$q = 'UPDATE `reference` SET `js_cache_short` = ? WHERE `id` = ?';
		$this->data->query($q, array($js_cache, $this->id));
	}	
	
	//Protected functions
	protected function check_id($func){
		//Return true if the id has been set, false and an error otherwise
		if($this->id == false){
			trigger_error("Pool_Reference->$func failed: No id defined", E_USER_WARNING);
			return false;
		} else return true;
	}

	protected function check_db($result, $func){
		//Return true if $result is not DB::Error, false and an error otherwise
		if(DB::isError($result)){
			trigger_error("Pool_Reference->$func failed: $this->data->last_query \n".$result->getMessage(), E_USER_WARNING);	
			return false;
		} else return true;
	}
	
	//static methods
	public static function new_hr_id($artifact_type, $title){
		//Creates a new hr_id based on the title of a reference
		//OK
		$data = &$GLOBALS['data'];
	
		$title = str_replace(' ', '', $title);
		if(strlen($title) > 18) $title = substr($title, 0, 17);
		$artifact_type = substr($artifact_type, 0, 3) . 'reference';
		$hr_name = $artifact_type.'_'.$title;
		$q = 'SELECT count(*) FROM `reference` WHERE `hr_id` LIKE ?';
		$result = $data->getOne($q, array($hr_name.'%'));
		if(!self::check_db($result, 'new_hr_id')) return false;
		$result++;
		$hr_num = sprintf("%02d", $result);
		return $hr_name.$hr_num;
	}
	
	public static function resolve_hr($hr_id, $table){
		//Turns a human readable ID into a numeric one
		$data = &$GLOBALS['data'];
		$q = "SELECT `id` FROM `$table` WHERE `$table`.`hr_id` = ?";
		$result = $data->getOne($q, array($hr_id));
		if(DB::isError($result)){
			trigger_error("Could not resolve hr_id '$hr_id' in '$table' table.", E_USER_WARNING);
			return false;
		} else {
			return $result;
		}
	}
	
	public static function id_to_hr($id){
		//Turns a numeric id into a human readable one
		$data = &$GLOBALS['data'];
		$q = "SELECT `hr_id` FROM `reference` WHERE `reference`.`id` = ?";
		$result = $data->getOne($q, array($id));
		if(DB::isError($result)){
			trigger_error("Could not resolve hr_id '$hr_id' in 'reference' table.", E_USER_WARNING);
			return false;
		} else {
			return $result;
		}		
	}
	
	public static function get_cached_js($reference_id){
		//Gets the cached javascript array for $reference_id
		$data = &$GLOBALS['data'];
		$q = 'SELECT `js_cache` FROM `reference` WHERE `reference`.`id` = ?';
		$result = $data->getOne($q, array($reference_id));
		if(DB::isError($result)){
			trigger_error("Could not find reference id $reference_id during Pool_Reference::get_cached_js.", E_USER_WARNING);
			return false;
		} else {
			return $result;
		}	
	}
	
	public static function get_cached_js_short($reference_id){
		//Gets the short version of the javascript array for $reference_id
		$data = &$GLOBALS['data'];
		$q = 'SELECT `js_cache_short` FROM `reference` WHERE `reference`.`id` = ?';
		$result = $data->getOne($q, array($reference_id));
		if(DB::isError($result)){
			trigger_error("Could not find reference id $reference_id during Pool_Reference::get_cached_js_short.", E_USER_WARNING);
			return false;
		} else {
			return $result;
		}	
	}
}
?>