<?PHP
//Pool Relationship Class .1
//This implements a pool relationship class that defines relationships between pool projects
//Requires the db_init and error_init files

require_once('db_init.php');

class Pool_Relationship{
	//Public variables (from db)
	public $id = false;				//ID of the relationship
	public $subject_id = false;		//ID of the subject
	public $predicate_id = false;	//ID of the predicate
	public $subject_type = false;	//Pool type of the subject
	public $predicate_type = false;	//Pool type of the predicate
	public $relationship = false;	//Relationship between the two projects
	public $description = '';		//Description of the relationship
	public $value = 0;				//Weight of the relationship
	public $recorder_id = '';		//Person who input the relationship

	//Protected db object
	//This is filled in the constructor
	protected $data = '';
					
	//Generic functions
	public function __construct($construct_var){
		//Generic constructor
		//$construct_var can be either a string or an array
		//If $construct_var is a string, it is assumed that you are loading a relationship from the db
		//If $construct_var is an array, it is assumed you are building a new relationship
		//Acutal implimentation of either is left up to the child class
		
		//Set up the db object
		$this->data = &$GLOBALS['data'];
				
		if(is_array($construct_var)) $this->new_relationship($construct_var);
		else {
			$this->id = $construct_var;
			$this->load_all();
		}
	}
	
	public function load_all(){
		//Loads all data from the db
		if(!$this->id) return false;
		$q = 'SELECT * FROM `relations` WHERE `id` = ?';
		$result = $this->data->getRow($q, array($this->id));
		foreach($result as $field){
			$this->$field = $field;
		}
	}
	
	public function new_relationship($form_data){
		//Creates a new relationship and adds it to the database
		//First determine the locations of the two projects.  If the pool type ends in ref, look in the reference table.  Otherwise look in the project table
		//Reference types are incomplete at this time
		
		$subject_table = strpos($form_data['subject_type'], 'ref') === false ? 'project' : 'reference';
		$predicate_table = strpos($form_data['predicate_type'], 'ref') === false ? 'project' : 'reference';
		
		//Create the objects names for the subject and predicate
		$subject_object_name = ucfirst($form_data['subject_type']).'_'.ucfirst($subject_table);
		$predicate_object_name = ucfirst($form_data['predicate_type']).'_'.ucfirst($predicate_table);

		//determine the weight of the relationship
		$weight = 1;
		if($subject_table == 'project'){
			if(Pool_Project::is_contributor($form_data['subject_type'], $form_data['subject_id'], $form_data['recorder_id'])) $weight = 2;
		}
		if($predicate_table == 'project'){
			if(Pool_Project::is_contributor($form_data['predicate_type'], $form_data['predicate_id'], $form_data['recorder_id'])) $weight = 5;
		}
		
		$relation_array = array($form_data['subject_id'], $form_data['predicate_id'], $form_data['subject_type'], $form_data['predicate_type'],
								$form_data['description'], $form_data['recorder_id'], $form_data['relationship'], $weight);
		
		//Insert the relationship into the database
		$q = 'INSERT INTO `relations` (`subject_id`, `predicate_id`, `subject_type`, `predicate_type`, `description`, `recorder_id`, `relationship`, `value`)
			VALUES (?, ?, ?, ?, ?, ?, ?, ?) 
			ON DUPLICATE KEY UPDATE `description` = VALUES(`description`), `value` = VALUES(`value`), `relationship` = VALUES(`relationship`)';
		$result = $this->data->query($q, $relation_array);
		if(!$this->check_db($result, 'new_relationship')) return false;
		
		//Create the objects for the subject and predicate, then update their JS caches
		if($subject_table == 'project'){
			$oSubject = new $subject_object_name($form_data['subject_id']);
			$oSubject->load_all();
			$oSubject->save_js();
			$oSubject->save_js_short();
		} else if($subject_table == 'reference'){
			$oSubject = new Pool_Reference($form_data['subject_id']);
			$oSubject->load_relations();
			$oSubject->load_reviews();
			$oSubject->set_rating_and_basis();
			$oSubject->load_ratings();
			$oSubject->save_js_short();
			$oSubject->save_js();
		}
		
		if($predicate_table == 'project'){
			$oPredicate = new $predicate_object_name($form_data['predicate_id']);
			$oPredicate->load_all();
			$oPredicate->save_js();
			$oPredicate->save_js_short();
		} else if ($predicate_table == 'reference'){
			$oPredicate = new Pool_Reference($form_data['predicate_id']);
			$oPredicate->load_relations();
			$oPredicate->load_reviews();
			$oPredicate->set_rating_and_basis();
			$oPredicate->load_ratings();
			$oPredicate->save_js_short();
			$oPredicate->save_js();				
		}
	}
	
	//Protected functions
	protected function check_id($func){
		//Return true if the id has been set, false and an error otherwise
		if($this->id == false){
			trigger_error("Pool_Project->$func failed: No id defined", E_USER_WARNING);
			return false;
		} else return true;
	}
	
	protected function check_db($result, $func){
		//Return true if $result is not DB::Error, false and an error otherwise
		if(DB::isError($result)){
			trigger_error("Pool_Project->$func failed: ".$this->data->last_query." \n".$result->getMessage(), E_USER_WARNING);	
			return false;
		} else return true;
	}
	//Static functions
}
?>