<?PHP
//release Class .1
//This implements a class for releases of all pool projects

require_once('db_init.php');
require_once('person.class.php');
require_once('pool_project.class.php');
require_once('review.class.php');

class Pool_Release{
	//Public variables (from db)
	public $id = false;				//ID of the release
	public $hr_id = '';				//Human readable ID of the release
	public $intent_id = '';			//ID of the intent this release came from
	public $approach_id = '';		//ID of the approach this release came from
	public $pool_type = '';			//Project type
	public $date = '';				//Date this release was last updated
	public $description = '';		//Full text description of the release
	public $uri_view = '';			//URI to view more on this release
	public $uri_source = '';		//URI to view source materials for this release
	public $requests = '';			//Requests for help on this release
	public $recorder_id = '';		//ID of the recorder of this release
	public $js_cache = '';			//Cached JS object for this release
	
	public $project_id = '';		//ID of the project this release is a part of
	//Array of contributors for this release
	//This will look like $contributors['contributor_id'] = array('id' => 39, 'written' = 'Contrib User', 'roles' => 'conceptual,technical');
	public $contributors = array();
	//Array of reviews for this release
	//This is an array of Review objects
	public $reviews = array();
	//Array of aspects for this release
	//This will look like $aspects['aspect_name'] = aspect_value
	public $aspects = array();
	//Array of ratings for this release
	//This will look like $rating['category'] = array('rating' => 6.55, 'basis' => 5);
	public $ratings = array();

	//Protected db object
	//This is filled in the constructor
	protected $data = '';
					
	//Abstract functions				

	//Generic functions
	public function __construct($construct_var){
		//Generic constructor
		//$construct_var can be either a string or an array
		//If $construct_var is a string, it is assumed that you are loading an release from the db
		//If $construct_var is an array, it is assumed you are adding a new release
		
		//Set up the db object
		$this->data = &$GLOBALS['data'];
		
		if(is_array($construct_var)) $this->new_release($construct_var);
		else{
			$this->id = $construct_var;
			$this->load_release();
		}
	}
	
	public function new_release($form_data){
		//saves a new release to the database
		//if $form_data['release_id'] != -1, update the old one instead.
		
		//Create the hr_id
		if($form_data['release_id'] == -1) $form_data['hr_id'] = $this->new_hr_id($form_data['pool_type'], $form_data['approach_id']);
		else $form_data['hr_id'] = Pool_Project::id_to_hr($form_data['release_id'], 'release');
		$this->hr_id = $form_data['hr_id'];
		
		//First get the project that $form_data['approach_id'] is part of
		$class_name = ucfirst($form_data['pool_type']).'_Project';
		$oProject = new $class_name(Pool_Project::resolve_artifact_id('approach', $form_data['approach_id']));
		$oProject->load_project();
		
		//Insert the release table data
		$q = 'INSERT INTO `release` (`hr_id`, `pool_type`, `intent_id`, `approach_id`, `date`, `description`, `uri_view`, `uri_source`, `requests`, `recorder_id`)
				VALUES (?,?,?,?,NOW(),?,?,?,?,?) 
				ON DUPLICATE KEY UPDATE `description` = VALUES(`description`), 
										`date` = VALUES(`date`),
										`uri_view` = VALUES(`uri_view`),
										`uri_source` = VALUES(`uri_source`),
										`requests` = VALUES(`requests`)';
		$release_data = array($form_data['hr_id'], $form_data['pool_type'], $form_data['intent_id'], $form_data['approach_id'], $form_data['description'], $form_data['uri_view'], $form_data['uri_source'], $form_data['requests'], $form_data['recorder_id']);
		$result = $this->data->query($q, $release_data);
		if(!$this->check_db($result, 'new_release')) return false;
		
		//Get back the id you just created
		$q = 'SELECT `id` FROM `release` WHERE `hr_id` = ?';
		$this->id = $this->data->getOne($q, array($form_data['hr_id']));
		
		//Connect contributors
		$q = 'DELETE FROM `person_artifact` WHERE `artifact_id` = ? AND `artifact_type` = ?';
		$result = $this->data->query($q, array($this->id, 'release'));
		if(!$this->check_db($result, 'new_release')) return false;
		
		$q = 'INSERT INTO `person_artifact` (`person_id`, `artifact_id`, `artifact_type`, `role`, `hr_link`)
				VALUES (?, ?, ?, ?, ?)
				ON DUPLICATE KEY UPDATE `hr_link` = `hr_link`';
		foreach($form_data['contributors'] as $person_id => $roles){
			$oPerson = new Pool_Person($person_id);
			foreach($roles as $role){
				$person_array = array($person_id, $this->id, 'release', $role, $oPerson->hr_id.'_'.$role);
				$result = $this->data->query($q, $person_array);
				if(!$this->check_db($result, 'new_release')) return false;
			}
		}
		
		//Connect aspects
		$q = 'DELETE FROM `aspects` WHERE `artifact_id` = ? AND `artifact_type` = ?';
		$result = $this->data->query($q, array($this->id, 'release'));
		if(!$this->check_db($result, 'new_intent')) return false;				
	
		$q = 'INSERT INTO `aspects` (`artifact_id`, `artifact_type`, `aspect`, `hr_link`, `value`)
				VALUES (?, ?, ?, ?, ?) ON DUPLICATE KEY UPDATE `value` = VALUES(`value`)';
		foreach($form_data['aspects'] as $aspect){
			if($aspect != ''){
				$aspect_array = array($this->id, 'release', $aspect, $form_data['hr_id'].'_'.$aspect, 1);
				$result = $this->data->query($q, $aspect_array);
				if(!$this->check_db($result, 'new_release')) return false;
			}
		}
		
		//Add custom rights, if they exist
		$aspect_array = array($this->id, 'release', 'custom-rights', $form_data['hr_id'].'_custom_rights', $form_data['custom_rights']);
		$result = $this->data->query($q, $aspect_array);
		if(!$this->check_db($result, 'new_release')) return false;
		
		//Connect project
		$q = 'INSERT INTO `project_artifact` (`project_id`, `artifact_id`, `artifact_type`, `hr_link`)
				VALUES (?, ?, ?, ?) ON DUPLICATE KEY UPDATE `hr_link` = `hr_link`';
		$result = $this->data->query($q, array($oProject->id, $this->id, 'release', $oProject->hr_id.'_'.$this->hr_id));
		if(!$this->check_db($result, 'new_release')) return false;		
		
		//Update project stage  count
		$oProject->recount_stages();
		
		//Update javascript caches
		$this->load_release();
		$this->save_js();
		$oProject->reload_all();
		$oProject->save_js();
		$oProject->save_js_short();	
	}
	
	public function load_release(){
		//Loads data from the release table
		
		//make sure the id has been set
		if(!$this->check_id('load_release')) return false;
		
		//Get the release table data
		$q = 'SELECT * FROM `release` WHERE `id` = ?';
		$result = $this->data->getRow($q, array($this->id));
		if(!$this->check_db($result, 'load_release')) return false;
		foreach($result as $key=>$col) $this->$key = $col;
		
		//Fill in the rest of the data from other tables
		$this->contributors = Pool_Project::get_contributor_list('release', $this->id);
		$this->project_id = Pool_Project::resolve_artifact_id('release', $this->id);
		$this->reviews = Pool_Project::get_reviews('release', $this->id);
		$this->aspects = Pool_Project::get_aspects('release', $this->id);
		$this->ratings = Pool_Project::get_ratings('release', $this->id);
	}
	
	public function write_js(){
		//writes the JS array for this intent
		
		require_once('smarty_init.php');
		$smarty = &$GLOBALS['smarty'];
		
		$smarty->assign('release', $this);
		$smarty->left_delimiter = '<{';
		$smarty->right_delimiter = '}>';
		$smarty->display('js_release_array.tpl');
		$smarty->left_delimiter = '{';
		$smarty->right_delimiter = '}';
	}
	
	public function save_js(){
		//writes the JS array for this review
		
		require_once('smarty_init.php');
		$smarty = &$GLOBALS['smarty'];
		
		$smarty->assign('release', $this);
		$smarty->left_delimiter = '<{';
		$smarty->right_delimiter = '}>';
		$js_cache = $smarty->fetch('js_release_array.tpl');
		$smarty->left_delimiter = '{';
		$smarty->right_delimiter = '}';
		
		$q = 'UPDATE `release` SET `js_cache` = ? WHERE `id` = ?';
		$this->data->query($q, array($js_cache, $this->id));
	}	
	
	//Protected functions
	protected function check_id($func){
		//Return true if the id has been set, false and an error otherwise
		if($this->id == false){
			trigger_error("Pool_Intent->$func failed: No id defined", E_USER_WARNING);
			return false;
		} else return true;
	}
	
	protected function check_db($result, $func){
		//Return true if $result is not DB::Error, false and an error otherwise
		if(DB::isError($result)){
			trigger_error("Pool_Intent->$func failed: ".$this->data->last_query." \n".$result->getMessage(), E_USER_WARNING);	
			return false;
		} else return true;
	}

	//static functions
	public static function new_hr_id($artifact_type, $approach_id){
		//Creates a new hr_id based on the intent_id
		$data = &$GLOBALS['data'];
	
		$oApproach = new Pool_Approach($approach_id);
		$q = 'SELECT count(*) FROM `release` WHERE `hr_id` LIKE ?';
		$result = $data->getOne($q, array($oApproach->hr_id.'%'));
		if(!self::check_db($result, 'new_hr_id')) return false;
		$result++;
		$hr_num = sprintf("%01d", $result);
		return $oApproach->hr_id.'.'.$hr_num;
	}	
}
?>