<?php

// $Id: rss10.inc,v 1.3 2001/05/20 17:58:02 edmundd Exp $
//
// A convenience class to make it easy to write RSS classes
// Edd Dumbill <mailto:edd+rsswriter@usefulinc.com>
//
// $Log: rss10.inc,v $
// Revision 1.3  2001/05/20 17:58:02  edmundd
// Final distribution tweaks.
//
// Revision 1.2  2001/05/20 17:41:30  edmundd
// Ready for distribution.
//
// Revision 1.1  2001/05/20 17:01:43  edmundd
// First functional draft of code working.
//
// Revision 1.1  2001/05/17 18:17:46  edmundd
// Start of a convenience library to help RSS1.0 creation
//

class RSSWriter {

function RSSWriter($uri, $title, $description, $meta=array()) {
	$this->chaninfo=array();
	$this->website=$uri;
	$this->chaninfo["link"]=$uri;
	$this->chaninfo["description"]=$description;
	$this->chaninfo["title"]=$title;
	$this->items=array();
	$this->modules=array("dc" => "http://purl.org/dc/elements/1.1/");
	// thanks James Mills for bugfix to this line
	$this->channelURI=POOL_HOME_URL;
	foreach ($meta as $key => $value) {
		$this->chaninfo[$key]=$value;
	}
}

function useModule($prefix, $uri) {
	$this->modules[$prefix]=$uri;
}

function setImage($imgURI, $imgAlt, $imgWidth=88, $imgHeight=31) {
	$this->image=array(
		"uri" => $imgURI, "title" => $imgAlt, "width" => $imgWidth,
		"height" => $imgHeight);
}

function addItem($uri, $title, $meta=array()) {
	$item=array("uri" => $uri, "link" => $uri, 
		"title" => $this->deTag($title));
	foreach ($meta as $key => $value) {
		if ($key == "description" || $key == "dc:description") {
			$value=$this->deTag($value);
		}
		$item[$key]=$value;
	}
	$this->items[]=$item;
}

function serialize() {
	$out = '';
	$out .= $this->preamble();
	$out .= $this->channelinfo();
	$out .= $this->image();
	$out .= $this->items();
	$out .= $this->postamble();
	return $out;
}

function deTag($in) {
	if(substr($in, 0, 8) == '<![CDATA') return $in;
  while(ereg('<[^>]+>', $in)) {
	$in=ereg_replace('<[^>]+>', '', $in);
  }
  return $in;
}

function preamble() {
	$out =  '<?xml version="1.0" encoding="iso-8859-1"?>
<rdf:RDF 
         xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
         xmlns="http://purl.org/rss/1.0/"
         xmlns:mn="http://usefulinc.com/rss/manifest/"
';
	foreach ($this->modules as $prefix => $uri) {
		$out .= "         xmlns:${prefix}=\"${uri}\"\n";
	}
	$out .= ">\n\n";
	return $out;
}

function channelinfo() {
	$out = '  <channel rdf:about="' .  $this->channelURI . '">
';
	$i=$this->chaninfo;
	foreach (array("title", "link", "dc:source", "description", "dc:language", "dc:publisher",
		"dc:creator", "dc:rights") as $f) {
		if (isset($i[$f])) {
			$out .= "    <${f}>" . htmlspecialchars($i[$f]) . "</${f}>\n";
		}
	}
	if (isset($this->image)) {
		$out .= "    <image rdf:resource=\"" . htmlspecialchars($this->image["uri"]) . "\" />\n";
	}
	$out .= "    <items>\n";
	$out .= "      <rdf:Seq>\n";
	foreach ($this->items as $i) {
		$out .= "        <rdf:li rdf:resource=\"" . htmlspecialchars($i["uri"]) . "\" />\n";
	}
	$out .= "      </rdf:Seq>\n";
	$out .= "    </items>\n";
	$out .= "  </channel>\n\n";
	return $out;
}

function image() {
	$out = '';
	if (isset($this->image)) {
		$out .=  "  <image rdf:about=\"" . htmlspecialchars($this->image["uri"]) . "\">\n";
    	$out .=  "     <title>" . htmlspecialchars($this->image["title"]) . "</title>\n";
    	$out .= "     <url>" . htmlspecialchars($this->image["uri"]) . "</url>\n";
    	$out .= "     <link>" . htmlspecialchars($this->website) . "</link>\n";
    	if ($this->chaninfo["description"]) {
   			$out .= "     <dc:description>" . htmlspecialchars($this->chaninfo["description"]) . 
   		 		"</dc:description>\n";
			$out .= "  </image>\n\n";
		}
	}
	return $out;	
}

function postamble() {
// 		$out = '  <rdf:Description rdf:ID="manifest">
//     <mn:channels>
//       <rdf:Seq>
//         <rdf:li rdf:resource="' . $this->channelURI . '" />
//       </rdf:Seq>
//     </mn:channels>
//   </rdf:Description>
	
	
	$out = '
</rdf:RDF>
';
	return $out;
}

function items() {
	$out = '';
	foreach ($this->items as $item) {
		$out .= "  <item rdf:about=\"" .  htmlspecialchars($item["uri"]) . "\">\n";
		foreach ($item as $key => $value) {
			if ($key!="uri") {
				if (is_array($value)) {
					foreach ($value as $v1) {
						if(substr($v1, 0, 8) == '<![CDATA')  $out .= "    <${key}>" . $v1 . "</${key}>\n";
						else $out .= "    <${key}>" . htmlspecialchars($v1) . "</${key}>\n";
					}
				} else {
					if(substr($value, 0, 8) == '<![CDATA')  $out .= "    <${key}>" . $value . "</${key}>\n";
					else $out .= "    <${key}>" . htmlspecialchars($value) . "</${key}>\n";
				}
			}
		}
		$out .= "  </item>\n\n";
	}
	return $out;
}

}

?>
