<?PHP
function get_article_array($id){
	//Takes an article id and returns an array with the article and 
	//it's XML data.
	$article_dbresult = get_post($id);
	$article_array = $article_dbresult->fetchRow();
	$var = parse_article($article_array['text']);
	$article_type = get_article_type($var);
	$article_array["xml"] = load_xml($var, $article_type);
	$article_array["xml_type"] = $article_type;
	return $article_array;
}

function parse_article($xml){
	//Creates a new DOM object containing the XML string passed.
	$dom = new DOMDocument();
	$dom->preserveWhiteSpace = FALSE;
	$dom->resolveExternals = TRUE;
	if(@!$dom->loadXML($xml)) die("read error.");//;header("Location: error.php?id=2");
	return $dom;
}

function get_article_type($dom){
	//Takes the DOM object created by parse_article and returns
	//the value of the class attribute of the post element
	//Assumes that there is only one post in the passed DOM
	$post_nodes = $dom->getElementsByTagName("post");
	$post_node = $post_nodes->item(0);
	return $post_node->getAttribute("class");
}

function get_single_node_value($dom, $name){
	//Takes the DOM object created by parse_article and
	//returns the value of a single node named $name.
	//If there is the possibility that there are multiple
	//nodes named $name in this DOM, use get_multiple_node_value
	$nodes = $dom->getElementsByTagName($name);
	$node = $nodes->item(0);
	return $node->textContent;
}

function get_multiple_node_values($dom, $name, $index=FALSE){
	//Takes the DOM object created by parse_article and
	//returns the value of a set of nodes named $name.
	//If you know there is only one node named $name
	//in this DOM, use get_single_node_value
	//Optionally, the array is indexed by the value
	//of an attribute named $index
	$nodes = $dom->getElementsByTagName($name);
	$value_array = array();
	foreach($nodes as $node){
		if($index){
			$key = $node->getAttribute($index);
			$value_array[$key] = $node->textContent;
		} else {
			array_push($node->textContent);
		}
	}
	return $value_array;
}

function xml_to_array($dom, $unique_nodes, $repeatable_nodes){
	//Turns the nodes in $dom named in $unique_nodes and
	//$repeatable_nodes into an associative array
	$xmlarray = array();
	foreach($unique_nodes as $node) $xmlarray[$node] = get_single_node_value($dom, $node);
	foreach($repeatable_nodes as $node) $xmlarray[$node[0]] = get_multiple_node_values($dom, $node[0], $node[1]);
	return $xmlarray;
}

function load_xml($dom, $article_type){
	//Creates the array of PHP variables Smarty will use based on 
	//$article_type from $dom
	$xml_description = map_xml($article_type);
	return xml_to_array($dom, $xml_description[0], $xml_description[1]);
}

function map_xml($xml_type){
	//Stores descriptions for each of the XML types	
	switch($xml_type){
		case "faculty":
			$unique_nodes = array("name", "address", "email", "url", "quote", "blurb", "image");
			$repeatable_nodes = array(array("section","name"));
		break;
		case "department":
			$unique_nodes = array("bodytext", "pagename");
			$repeatable_nodes = array(array("section", "name"));
		break;
	}
	return array($unique_nodes, $repeatable_nodes);
}

function create_xmldoc($data_array){
	$imp = new DomImplementation();
	$dtd = $imp->createDocumentType("page", "", "/tech/character-entities.dtd");
	$dom = $imp->createDocument("","",$dtd);
	
	$post_element = $dom->createElement("post");
	$post_element->setAttribute("class", $data_array[xml_type]);
	$dom->appendChild($post_element);
	
	$xml_description = map_xml($data_array[xml_type]);
	
	foreach($xml_description[0] as $nodename){
		if (iconv('UTF-8', 'UTF-8', $data_array[$nodename]) != $data_array[$nodename]) { 
			header("Location: error.php?id=7");
		}
		$element = $dom->createElement($nodename, $data_array[$nodename]);
		$post_element->appendChild($element);
	}
	
	foreach($xml_description[1] as $nodename){
		foreach($data_array[$nodename[0]] as $nodedata){
			if (iconv('UTF-8', 'UTF-8', $nodedata[$nodename[1]]) != $nodedata[$nodename[1]]) { 
				header("Location: error.php?id=7");
			}
			$element = $dom->createElement($nodename[0], $nodedata[value]);
			$element->setAttribute($nodename[1], $nodedata[$nodename[1]]);
			$post_element->appendChild($element);
		}
	}
	
 	if(!$dom->saveXML()) header("Location: error.php?id=7");;
	$xml_string = $dom->saveXML() or die("xml error");
	
	$dom_test = new DOMDocument();
	$dom_test->preserveWhiteSpace = FALSE;
	$dom_test->resolveExternals = TRUE;
	if(!$dom_test->loadXML($xml_string)) {
		//print_r($xml_string);
		//die();
		header("Location: error.php?id=7");
	}
	else return $dom;
}
?>