/**
 * @author johnbell
 */
function PersonPanel(titleText){
	var panel = new PoolPanel(titleText);
	panel.panelHandle.className = 'personPanel';
	
	//methods
	panel.loadData = function(dataArray){
		//add the panelId to the dataArray for future reference and store the dataArray in the panel
		dataArray.panelId = panel.panelHandle.panelId;
		panel.panelHandle.json = dataArray;
		panel.panelHandle.panelType = 'person';
		
		//Build the tabBox
		if(dataArray.person.id == browsers.readCookie('poolid') || browsers.readCookie('poollevel') == 2){
			var contentDiv = new tabSet(['general', 'projects', 'reviews of', 'reviews by', 'account']);
		} else {
			var contentDiv = new tabSet(['general', 'projects', 'reviews of', 'reviews by']);
		}
		contentDiv.tabBox.style.display = 'block';
		panel.panelContent.appendChild(contentDiv.tabBox);

		//Fill in the data
		panel.loadGeneral(contentDiv.tabBox.content['general'], dataArray.person);
		panel.loadReviews(contentDiv.tabBox.content['reviews by'], dataArray.reviews_by);
		panel.loadReviews(contentDiv.tabBox.content['reviews of'], dataArray.reviews_of);
		panel.loadProjects(contentDiv.tabBox.content['projects'], dataArray.projects, dataArray.person.id);
		if(dataArray.person.id == browsers.readCookie('poolid') || browsers.readCookie('poollevel') == 2){
			panel.loadAccount(contentDiv.tabBox.content['account'], dataArray.person, panel.panelHandle.panelId);
		}
		
		//Register panel with state manager
		workingBasin.state.panels.push(['person', dataArray.person.id]);
		if(init == false) ProtoHistoryObj.savePool(workingBasin.state);
	}
	
	panel.loadGeneral = function(targetDiv, personArray){
		//Load the general data about this person
		
		targetDiv.innerHTML = panelContent.person_general;
		insertXMLTagData(targetDiv, 'ppWritten', personArray.written);
		if(personArray.uri != '') insertXMLTagData(targetDiv, 'ppURI', '<a href="'+personArray.uri+'" target="_blank">Home Page</a>');
		if(personArray.email != '') insertXMLTagData(targetDiv, 'ppEmail', '<a href="mailto:'+personArray.email+'">Email</a>');
		insertXMLTagData(targetDiv, 'ppThemes', personArray.themes);
		insertXMLTagData(targetDiv, 'ppBio', personArray.description);
		
		if(!personArray.thumb_id || personArray.thumb_id == '') personArray.thumb_id = 'media/person_icon.jpg';
		else personArray.thumb_id = 'image.php?id='+personArray.thumb_id;
		insertXMLSrc(targetDiv, 'ppIcon', personArray.thumb_id);

		//insert aptitudes
		var aptitudes = document.createElement('div');
		for(var category in personArray.ratings){
			if(personArray.ratings[category].self_rating != '' && personArray.ratings[category].self_rating > 0){
				var aptitudeDiv = aptitudes.appendChild(document.createElement('div'));
				var label = aptitudeDiv.appendChild(document.createElement('span'));
				label.innerHTML = category.substring(0,1).toUpperCase() + category.substring(1, category.length) + ': ' + personArray.ratings[category].self_rating;
				var meter = aptitudeDiv.appendChild(document.createElement('div'));
				meter.className = 'ppPersonMeterEmpty';
				var fill = meter.appendChild(document.createElement('div'));
				fill.className = 'ppPersonMeterFull';
				fill.style.width = ((personArray.ratings[category].self_rating / 10) * 100) + '%';
				fill.style.height = '22px';  //I don't know why I have to set this here, when I did it in just the css it didn't work
				if(personArray.ratings[category].rating != ''){
					var rdfValue = Math.round(Math.abs(personArray.ratings[category].rating - personArray.ratings[category].self_rating));
					var rdfImage = meter.appendChild(document.createElement('img'));
					rdfImage.className = 'ppRDF';
					insertXMLSrc(meter, 'ppRDF', 'media/rdf'+rdfValue+'.png');
				}
				aptitudeDiv.appendChild(document.createElement('br'));
			}
		}
		insertXMLTagChild(targetDiv, 'ppRatings', aptitudes);		
	}
	
	panel.loadReviews = function(targetDiv, reviewArray){
	//Loads data from reviewArray into targetDiv using a template defined in review.fragment.tpl (looping)
		var assembled = "";
		for(var revID in reviewArray){
			//load the fragment to the stage so we can work with it
			var stageName = reviewArray[revID].pool_type+'_'+reviewArray[revID].artifact_type+'_review_frag';
			var workingFrag = loadFragmentToStage(stageName);
			//send data to the stage
			insertXMLTagDataFragment('ppReviewRating', reviewArray[revID].rating, workingFrag);
			insertXMLTagDataFragment('ppReviewAuthor', reviewArray[revID].recorder_written, workingFrag);
			insertXMLTagDataFragment('ppReview', reviewArray[revID].description, workingFrag);

			insertXMLMouseUp(workingFrag, 'ppReviewAuthor', 'var tempFish = new PersonFish("'+reviewArray[revID].recorder_id+'","'+reviewArray[revID].recorder_written+'");tempFish.launchPanel();');

			var hrString = new String(reviewArray[revID].hr_link);
			var hrSplit = hrString.split('_');
			var version = hrSplit[hrSplit.length - 1];
			var artifactType = reviewArray[revID].artifact_type.substring(0,1).toUpperCase() + reviewArray[revID].artifact_type.substring(1,reviewArray[revID].artifact_type.length);
			if(hrSplit.length < 5) version = reviewArray[revID].artifact_title;
			else version = reviewArray[revID].artifact_title + ' ' + version;
			insertXMLTagDataFragment('ppVersion', version, workingFrag);
			
			if(reviewArray[revID].subratings.conceptual && reviewArray[revID].subratings.conceptual != ''){
				insertXMLTagDataFragment('ppConceptualRating', reviewArray[revID].subratings.conceptual.rating, workingFrag);
				insertXMLTagDataFragment('ppConceptual', reviewArray[revID].subratings.conceptual.description, workingFrag);
			}

			if(defined(reviewArray[revID].subratings.perceptual) && reviewArray[revID].subratings.perceptual != ''){
				insertXMLTagDataFragment('ppPerceptualRating', reviewArray[revID].subratings.perceptual.rating, workingFrag);
				insertXMLTagDataFragment('ppPerceptual', reviewArray[revID].subratings.perceptual.description, workingFrag);	
			}

			if(reviewArray[revID].subratings.technical && reviewArray[revID].subratings.technical != ''){
				insertXMLTagDataFragment('ppTechnicalRating', reviewArray[revID].subratings.technical.rating, workingFrag);
				insertXMLTagDataFragment('ppTechnical', reviewArray[revID].subratings.technical.description, workingFrag);			
			}
			
			if(reviewArray[revID].subratings.utility && reviewArray[revID].subratings.utility != ''){
				insertXMLTagDataFragment('ppUtilityRating', reviewArray[revID].subratings.utility.rating, workingFrag);
				insertXMLTagDataFragment('ppUtility', reviewArray[revID].subratings.utility.description, workingFrag);			
			}
			
			
//			if(reviewArray[revID].subratings.conceptual && reviewArray[revID].subratings.conceptual != ''){
//				insertXMLTagDataFragment('ppConceptualRating', reviewArray[revID].subratings.conceptual.rating, workingFrag);
//				insertXMLTagDataFragment('ppConceptual', reviewArray[revID].subratings.conceptual.description, workingFrag);
//			}
//			if(reviewArray[revID].artifact_type == 'approach' || reviewArray[revID].artifact_type == 'release' || reviewArray[revID].artifact_type == 'project'){
//				if(defined(reviewArray[revID].subratings.perceptual) && reviewArray[revID].subratings.perceptual != ''){
//					insertXMLTagDataFragment('ppPerceptualRating', reviewArray[revID].subratings.perceptual.rating, workingFrag);
//					insertXMLTagDataFragment('ppPerceptual', reviewArray[revID].subratings.perceptual.description, workingFrag);	
//				}
//
//				if(reviewArray[revID].subratings.technical && reviewArray[revID].subratings.technical != ''){
//					insertXMLTagDataFragment('ppTechnicalRating', reviewArray[revID].subratings.technical.rating, workingFrag);
//					insertXMLTagDataFragment('ppTechnical', reviewArray[revID].subratings.technical.description, workingFrag);			
//				}
//			}
			
			
			
			
			
			
			//Append working fragment to assembled string
			targetDiv.appendChild(workingFrag);
		}	
	}
	
	panel.loadProjects = function(targetDiv, projectsArray, personID){
	//loads project data into the person panel
		for(projID in projectsArray){
			var projectDiv = targetDiv.appendChild(document.createElement('div'));
			switch(projectsArray[projID].pool_type){
				case 'art':		var projectFish = new ArtFish(projectsArray[projID]);		break;
				case 'code':	var projectFish = new CodeFish(projectsArray[projID]);		break;
				default: '';
			}
			//turn off some functions of fish that don't make sense in the panel
			projectFish.fishDiv.style.position = 'static';
			//browsers.setFloat(projectFish.fishDiv, 'left');
			//projectFish.fishDiv.style.float = 'left';
			projectFish.fishDiv.onmouseover = '';
			projectFish.fishDiv.onmouseout = '';
			projectFish.fishDiv.className = 'clickable';
			projectDiv.appendChild(projectFish.fishDiv);
			var subratingCount = 0;
			for(category in projectsArray[projID].subratings){
				if(projectsArray[projID].subratings[category].rating > 0){
					subratingCount++;
					var categoryIcon = document.createElement('img');
					categoryIcon.src = siteBaseDir + 'media/person_' +category+'_icon.gif';
					categoryIcon.altText = category;
					categoryIcon.title = category;
					categoryIcon.style.verticalAlign = 'middle';
					projectFish.fishDiv.appendChild(categoryIcon);
					var categoryRating = document.createElement('span');
					categoryRating.innerHTML = projectsArray[projID].subratings[category].rating;
					
					var green = parseInt(((projectsArray[projID].subratings[category].rating)/10)*255); //How good the rating is.
					var red = parseInt(((10-projectsArray[projID].subratings[category].rating)/10)*255); //How bad the rating is.
					var blue = parseInt(((5-projectsArray[projID].subratings[category].basis)/5)*128); //How reliable the sample basis is.
					categoryRating.style.color = 'rgb('+red+','+green+','+blue+')';

					projectFish.fishDiv.appendChild(categoryRating);
				}
			}
			if(subratingCount == 0){
				projectFish.fishDiv.appendChild(document.createTextNode('  (unrated)'));
			}
			//add other contributors
			var contribCount = 0;
			var contribString = '(with ';
			var contribs = new Array();

			for(contribID in projectsArray[projID].contributors){
				if(contribID != personID) {
					contribCount++;
					contribs.push(projectsArray[projID].contributors[contribID].written);
				}
			}					
			if(contribCount > 0){
				contribString += contribs.join(', ');
				contribString += ')';
				var contribSpan = document.createElement('div');
				contribSpan.className = 'lm10';
				contribSpan.innerHTML = contribString;
				projectDiv.appendChild(contribSpan);						
			}
			var rule = projectDiv.appendChild(document.createElement('hr'));
			rule.style.width = '80%';
		}
	}
	
	panel.loadAccount = function(targetDiv, personArray, panelID){
		//Loads the account settings page if the person is logged in
		targetDiv.innerHTML = panelContent.account;
		var formNodes = targetDiv.getElementsByTagName('form');
		var accountForm = formNodes[0];
		accountForm.onsubmit = ajax.parseForm;
		
		//set the ratings
		for(var category in personArray.ratings){
			if(personArray.ratings[category].self_rating != '' && personArray.ratings[category].self_rating > 0){
				var fieldName = category+'Self';
				accountForm[fieldName].value = personArray.ratings[category].self_rating;
			}
		}
		//set profile info
		//insertXMLTagData(targetDiv, 'ppThumbUpload', '<a href="'+siteBaseDir+'image_upload.php?id='+personArray.id+"&thumb_type=person\" target='_new'>+ Upload Icon</a>");
		
		insertXMLMouseUp(targetDiv, 'ppThumbUpload', 'openThumbUpload("'+personArray.id+'","person","'+panelID+'")');

		
		accountForm.email.value = personArray.email;
		accountForm.uri.value = personArray.uri;
		accountForm.themes.value = personArray.themes;
		accountForm.description.value = personArray.description;
		accountForm.userID.value = personArray.id;
		accountForm.target.value = siteBaseDir+'ajaxHandlers/person_handler.js.php';
		accountForm.panelId.value = panel.panelHandle.panelId;
	}
	
	
	return panel;
}

Loader.registerFile('PersonPanel.js');
