/**

 *  DivPlus Scrolling accessory

 *  Part of DivPlus (http://divplus.novomancy.org/)

 *

 *  Function which adds scrollbars to any DivPlus div, making it scrollable inside a

 *  window defined by clipping parameters.

 *

 *  @author	john.bell@REMOVEME.umit.maine.edu

 *  @version 1.3.20040526

 *  @uses divplus.js

 *  @uses divplusdragger.js

 *  @uses defined.js

 *  @uses browsers.js

 *

 *  @argument	scrollingDiv		obj		DivPlus object which contains the div you want to make scrollable

 *  @argument	scrollDirection		str		"h" or "v" to add only horizontal or vertical scrollbars.

 *											"both" allows full scrolling in both directions.

 *  @argument	windowSize			arr		Associative array with the height and width of the clipping window, in pixels

 *  @argument	appearance			arr		Associative array specifying the appearance of the individual components which

 *											make up the scrollbars.  The properties below are valid, though not all are

 *											required.  In general the defaults for each property are width=10, height=10,

 *											color=transparent, and src=''.  So you should define either color or src,

 *											at the very least.

 *												left		arr		Associative array describing the left arrow div

 *																		src:    Source of the image to use for the left arrow

 *																		width:  Width of the left arrow div

 *																		color:	Background color of the left arrow div, decimal rgb array

 *												right		arr		Associative array describing the right arrow div

 *																		src:    Source of the image to use for the right arrow

 *																		width:  Width of the right arrow div

 *																		color:	Background color of the right arrow div, decimal rgb array

 *												up			arr		Associative array describing the up arrow div

 *																		src:    Source of the image to use for the up arrow

 *																		height: Height of the up arrow div

 *																		color:	Background color of the up arrow div, decimal rgb array

 *												down		arr		Associative array describing the down arrow div

 *																		src:    Source of the image to use for the down arrow

 *																		height: Height of the down arrow div

 *																		color:	Background color of the down arrow div, decimal rgb array

 *												handleV		arr		Associative array describing the vertical handle div

 *																		src:    Source of the image to use for the vertical handle

 *																		width:  Width of the vertical handle div

 *																		color:	Background color of the vertical handle div, decimal rgb array

 *												handleH		arr		Associative array describing the horizontal handle div

 *																		src:    Source of the image to use for the horizontal handle

 *																		height: Height of the horizontal handle div

 *																		color:	Background color of the horizontal handle div, decimal rgb array

 *												channelV	arr		Associative array describing the vertical channel div

 *																		src:	Source of the image to use for the horizontal channel

 *																		width:	Width of the channel, up and down arrows, and handleV

 *																		color:	Background color of the vertical channel div, decimal rgb array

 *																		force:	Set to "Yes" to force the display of scroll bars even when they

 *																				aren't needed.

 *												channelH	arr		Associative array describing the horiztonal channel div

 *																		src:	Source of the image to use for the horizontal channel

 *																		height:	Height of the channel, left and right arrows, and handleH

 *																		color:	Background color of the horizontal channel div, decimal rgb array

 *																		force:	Set to "Yes" to force the display of scroll bars even when they

 *																				aren't needed.

 *

 *  @returns 	nothing, but does add a new property to the DivPlus object specified: [DivPlusObject].scroller.

 *				This object has the following externally useful methods:

 *					[DivPlusObject].scroller.setHTML(html)			Changes the contents of the scrolling div

 *																	and handles updating all scroll control objects

 *					[DivPlusObject].scroller.resetSize()			Updates control objects when the content of

 *																	the scrolling div changes.  Should only really

 *																	be needed by the scroller.setHTML method, but

 *																	may be useful at other times as well.

 */



function DivPlusScroller(scrollingDiv,scrollDirection,windowSize, appearance){

	scrollingDiv.scroller = this;

	this.scrollDirection = scrollDirection;

	this.appearance = appearance;

	this.windowSize = windowSize;

	this.contentSize = scrollingDiv.getSize();

	if(scrollingDiv.getWidth() + scrollingDiv.getPadding().left + scrollingDiv.getPadding().right < this.windowSize.width){

		scrollingDiv.setWidth(windowSize.width - scrollingDiv.getPadding().left - scrollingDiv.getPadding().right);

	}

	if(scrollingDiv.getHeight() + scrollingDiv.getPadding().top + scrollingDiv.getPadding().bottom < this.windowSize.height){

		scrollingDiv.setHeight(windowSize.height - scrollingDiv.getPadding().top - scrollingDiv.getPadding().bottom);

	}

	

	this.size = scrollingDiv.getSize();

	this.windowSize.width += scrollingDiv.getPadding().left + scrollingDiv.getPadding().right;

	this.windowSize.height += scrollingDiv.getPadding().top + scrollingDiv.getPadding().bottom;



	this.scrollControl = "0";

	browsers.addEventListener("mousedown", scrollingDiv.element, browsers.stopBubbling, false);

	

	this.images = {	left:{width:10,height:10},

					right:{width:10,height:10},

					up:{width:10,height:10},

					down:{width:10,height:10},

					handleV:{width:10,height:10},

					handleH:{width:10,height:10},

					channelV:{width:10},

					channelH:{height:10}};

					

	for(divName in appearance){

		for(styleProp in appearance[divName]){

			this.images[divName][styleProp] = appearance[divName][styleProp];

		}

		if(divName=="channelV"){

			if(defined(appearance.channelV.width)){

				this.images.handleV.width = appearance.channelV.width;

				this.images.up.width = appearance.channelV.width;

				this.images.down.width = appearance.channelV.width;

			}

		}

		if(divName=="channelH"){

			if(defined(appearance.channelH.height)){

				this.images.handleH.height = appearance.channelH.height;

				this.images.left.height = appearance.channelH.height;

				this.images.right.height = appearance.channelH.height;

			}

		}

	}



	if(scrollDirection=="h" || scrollDirection=="both")	var scrollBoxHeight = windowSize.height + this.images.channelH.height;

	else var scrollBoxHeight = windowSize.height;

	if(scrollDirection=="v" || scrollDirection=="both") var scrollBoxWidth = windowSize.width + this.images.channelV.width;

	else var scrollBoxWidth = windowSize.width;

	

	this.scrollBox = new DivPlus(scrollingDiv.objectID+"scrollBox",{top: scrollingDiv.getTop(),

																	left: scrollingDiv.getLeft(),

																	//height:scrollBoxHeight,

																	//width:scrollBoxWidth,

																	//overflow:"hidden",

																	container: scrollingDiv.element.parentNode.id});

	this.scrollBox.element.appendChild(scrollingDiv.element);

	scrollingDiv.setTop(0);

	scrollingDiv.setLeft(0);											 

	this.scrollSpeed = 31;

	scrollingDiv.setClipping({top:0,left:0,right:windowSize.width,bottom:windowSize.height});

	

	if(scrollDirection=="h" || scrollDirection=="both"){

		//Build the horizontal scrollbars

		

		this.channelHWidth = this.windowSize.width - (this.images.left.width + this.images.right.width);

		this.handleHWidth = Math.floor((this.channelHWidth/ this.size.width) * this.channelHWidth);

		if(this.handleHWidth < 10) this.handleHWidth = 10;

		this.channelHFloatLimit = this.channelHWidth - this.handleHWidth;



		//Set styles for control divs

		var leftArrowStyle = {	left: 0,

								top: windowSize.height,

								width: this.images.left.width,

								height: this.images.left.height,

								fontSize: 1,

								container: this.scrollBox.objectID,

								onmouseup:"divList['"+scrollingDiv.objectID+"'].scroller.scrollControl=0;",

								onmouseout:"divList['"+scrollingDiv.objectID+"'].scroller.scrollControl=0;",

								onmousedown:"divList['"+scrollingDiv.objectID+"'].scroller.scrollControl=1;divList['"+scrollingDiv.objectID+"'].scroller.scrollLeft(e);"};

		if(defined(this.images.left.color)) leftArrowStyle.backgroundColor = this.images.left.color;

		if(defined(this.images.left.src)) leftArrowStyle.html = "<img height='"+this.images.left.height+"' width='"+this.images.left.width+"' src='"+this.images.left.src+"'>";



		var rightArrowStyle = {	left: windowSize.width - this.images.right.width,

								top: windowSize.height,

								width: this.images.right.width,

								height: this.images.right.height,

								fontSize: 1,

								container: this.scrollBox.objectID,

								onmouseup:"divList['"+scrollingDiv.objectID+"'].scroller.scrollControl=0;",

								onmouseout:"divList['"+scrollingDiv.objectID+"'].scroller.scrollControl=0;",

								onmousedown:"divList['"+scrollingDiv.objectID+"'].scroller.scrollControl=1;divList['"+scrollingDiv.objectID+"'].scroller.scrollRight(e);"};

		if(defined(this.images.right.color)) rightArrowStyle.backgroundColor = this.images.right.color;

		if(defined(this.images.right.src)) rightArrowStyle.html = "<img height='"+this.images.right.height+"' width='"+this.images.right.width+"' src='"+this.images.right.src+"'>";



		var channelHStyle = {	left: this.images.left.width,

								top: windowSize.height,

								width: this.channelHWidth,

								height: this.images.handleH.height,

								fontSize: 1,

								container: this.scrollBox.objectID};

		if(defined(this.images.channelH.color)) channelHStyle.backgroundColor = this.images.channelH.color;

		if(defined(this.images.channelH.src)) channelHStyle.html = "<img height='"+this.images.channelH.height+"' width='"+this.channelHwidth+"' src='"+this.images.channelH.src+"'>";

		

		var handleHStyle = {	left:0,

								top: 0,

								width: this.handleHWidth,

								height: this.images.handleH.height,

								fontSize: 1,

								container: scrollingDiv.objectID+"channelH",

								onmouseup:"divList['"+scrollingDiv.objectID+"'].scroller.scrollControl = 0;",

								onmousedown:"divList['"+scrollingDiv.objectID+"'].scroller.scrollControl = 1;divList['"+scrollingDiv.objectID+"'].scroller.dragScrollH(mouseY, mouseX, e);"};

		if(defined(this.images.handleH.color)) handleHStyle.backgroundColor = this.images.handleH.color;

		if(defined(this.images.handleH.src)) handleHStyle.html = "<img height='"+this.images.handleH.height+"' width='"+this.handleHWidth+"' src='"+this.images.handleH.src+"'>";



		

		//Build controls

		this.leftArrow = new DivPlus(scrollingDiv.objectID+"scrollLeft",leftArrowStyle);

		this.rightArrow = new DivPlus(scrollingDiv.objectID+"scrollRight",rightArrowStyle);

		this.channelH = new DivPlus(scrollingDiv.objectID+"channelH",channelHStyle);

		this.handleH = new DivPlus(scrollingDiv.objectID+"handleH",handleHStyle);

		new DivPlusDragger(this.handleH, "h", {left:0, right:this.channelHFloatLimit});



		

		//Methods dealing with horizontal scrolling

		this.hideHorizontal = function(){

			this.leftArrow.setDisplay("none");

			this.rightArrow.setDisplay("none");

			this.channelH.setDisplay("none");

			this.channelH.setDisplay("none");

		}

		this.showHorizontal = function(){

			this.leftArrow.setDisplay("block");

			this.rightArrow.setDisplay("block");

			this.channelH.setDisplay("block");

			this.channelH.setDisplay("block");			

		}

				

		if(this.contentSize.width <= this.windowSize.width && this.images.channelH.force != "Yes"){

			this.hideHorizontal();

		}

		

		this.scrollLeft = function(e){

			if(e || window.event) browsers.stopBubbling(e);

			if((scrollingDiv.getLeft() + this.scrollSpeed < 0) && this.scrollControl){

				scrollingDiv.setLeft(scrollingDiv.getLeft() + this.scrollSpeed);

				var oldClip = scrollingDiv.getClipping();

				scrollingDiv.setClipping({	top:oldClip.top,

											left:oldClip.left - this.scrollSpeed, 

											right:oldClip.right - this.scrollSpeed, 

											bottom:oldClip.bottom});

				var relativePosition = Math.abs(scrollingDiv.getLeft()) / this.size.width;

				this.handleH.setLeft(Math.floor(this.channelHWidth * relativePosition));

				setTimeout("divList['"+scrollingDiv.objectID+"'].scroller.scrollLeft()",100);

			} else if (scrollingDiv.getLeft() + this.scrollSpeed >= 0){

				this.handleH.setLeft(0);

				scrollingDiv.setLeft(0);

				var oldClip = scrollingDiv.getClipping();

				scrollingDiv.setClipping({	top:oldClip.top, 

											left:0, 

											right: this.windowSize.width, 

											bottom:oldClip.bottom});

				this.scrollControl = 0;

			}

			return false;

		}		

		this.scrollRight = function(e){

			if(e || window.event) browsers.stopBubbling(e);

			if((scrollingDiv.getLeft() + this.size.width - this.scrollSpeed > this.windowSize.width) && this.scrollControl){

				scrollingDiv.setLeft(scrollingDiv.getLeft() - this.scrollSpeed);

				var oldClip = scrollingDiv.getClipping();

				scrollingDiv.setClipping({	top:oldClip.top, 

											left:oldClip.left + this.scrollSpeed, 

											right:oldClip.right + this.scrollSpeed, 

											bottom:oldClip.bottom});

				var relativePosition = Math.abs(scrollingDiv.getLeft()) / this.size.width;

				this.handleH.setLeft(Math.floor(this.channelHWidth * relativePosition));

				setTimeout("divList['"+scrollingDiv.objectID+"'].scroller.scrollRight()",100);

			} else  if (scrollingDiv.getLeft() + this.size.width - this.scrollSpeed <= this.windowSize.width){

				this.handleH.setLeft(this.channelHFloatLimit);

				scrollingDiv.setLeft(this.windowSize.width - this.size.width);

				var oldClip = scrollingDiv.getClipping();

				scrollingDiv.setClipping({	top:oldClip.top, 

											left:this.size.width - this.windowSize.width, 

											right: this.size.width, 

											bottom:oldClip.bottom});

				this.scrollControl = 0;

			}

			return false;

		}

		this.dragScrollH = function(mouseY, mouseX, e){

			if(e || window.event) browsers.stopBubbling(e);

			if(this.scrollControl){

				var relativePosition = Math.abs(this.handleH.getLeft() / this.channelHFloatLimit);

				var oldClip = scrollingDiv.getClipping();

				var effectiveWidth = this.size.width - this.windowSize.width;

				if(relativePosition * effectiveWidth > this.size.width){

					scrollingDiv.setLeft((this.size.width - this.windowSize.width) * -1)

					scrollingDiv.setClipping({	top:oldClip.top,

												left: this.size.width - this.windowSize.width,

												right: this.size.width,

												bottom: oldClip.bottom});

				} else {

				scrollingDiv.setLeft(Math.floor(relativePosition * effectiveWidth * -1));

				var oldClip = scrollingDiv.getClipping();

				scrollingDiv.setClipping({	top:oldClip.top, 

											left:Math.ceil(relativePosition * effectiveWidth), 

											right:Math.ceil(relativePosition * effectiveWidth) + this.windowSize.width, 

											bottom:oldClip.bottom});

				}

				setTimeout("divList['"+scrollingDiv.objectID+"'].scroller.dragScrollH()", 10);

			} else {

				this.scrollControl = 0;

			}

			return false;

		}

	}

	

	if(scrollDirection=="v" || scrollDirection=="both"){

		//Build the vertical scrollbars

		

		this.channelVHeight = this.windowSize.height - (this.images.up.height + this.images.down.height);

		this.handleVHeight = Math.floor((this.channelVHeight/ this.size.height) * this.channelVHeight);

		if(this.handleVHeight < 10) this.handleVHeight = 10;

		this.channelVFloatLimit = this.channelVHeight - this.handleVHeight;



		//Set styles for control divs

		var upArrowStyle = {	left: windowSize.width,

								top: 0,

								width: this.images.up.width,

								height: this.images.up.height,

								fontSize: 1,

								container: this.scrollBox.objectID,

								onmouseup:"divList['"+scrollingDiv.objectID+"'].scroller.scrollControl=0;",

								onmouseout:"divList['"+scrollingDiv.objectID+"'].scroller.scrollControl=0;",

								onmousedown:"divList['"+scrollingDiv.objectID+"'].scroller.scrollControl=1;divList['"+scrollingDiv.objectID+"'].scroller.scrollUp(e);"};

		if(defined(this.images.up.color)) upArrowStyle.backgroundColor = this.images.up.color;

		if(defined(this.images.up.src)) upArrowStyle.html = "<img height='"+this.images.up.height+"' width='"+this.images.up.width+"' src='"+this.images.up.src+"'>";



		var downArrowStyle = {	left: windowSize.width,

								top: windowSize.height - this.images.down.height,

								width: this.images.down.width,

								height: this.images.down.height,

								fontSize: 1,

								container: this.scrollBox.objectID,

								onmouseup:"divList['"+scrollingDiv.objectID+"'].scroller.scrollControl=0;",

								onmouseout:"divList['"+scrollingDiv.objectID+"'].scroller.scrollControl=0;",

								onmousedown:"divList['"+scrollingDiv.objectID+"'].scroller.scrollControl=1;divList['"+scrollingDiv.objectID+"'].scroller.scrollDown(e);"};

		if(defined(this.images.down.color)) downArrowStyle.backgroundColor = this.images.down.color;

		if(defined(this.images.down.src)) downArrowStyle.html =  "<img height='"+this.images.down.height+"' width='"+this.images.down.width+"' src='"+this.images.down.src+"'>";

		

		var channelVStyle = { 	left: windowSize.width,

								top: this.images.up.height,

								width: this.images.handleV.width,

								height: this.channelVHeight,

								fontSize: 1,

								container: this.scrollBox.objectID};

		if(defined(this.images.channelV.color)) channelVStyle.backgroundColor = this.images.channelV.color;

		if(defined(this.images.channelV.src)) channelVStyle.html = "<img height='"+this.channelVHeight+"' width='"+this.images.channelV.width+"' src='"+this.images.channelV.src+"'>";

		

		var handleVStyle = {	left: 0,

								top: 0,

								width: this.images.handleV.width,

								height: this.handleVHeight,

								fontSize: 1,

								container: scrollingDiv.objectID+"channelV",

								onmouseup:"divList['"+scrollingDiv.objectID+"'].scroller.scrollControl = 0;",

								onmousedown:"divList['"+scrollingDiv.objectID+"'].scroller.scrollControl = 1;divList['"+scrollingDiv.objectID+"'].scroller.dragScrollV(mouseY, mouseX, e);"};

		if(defined(this.images.handleV.color)) handleVStyle.backgroundColor = this.images.handleV.color;

		if(defined(this.images.handleV.src)) handleVStyle.html = "<img height='"+this.handleVHeight+"' width='"+this.images.handleV.width+"' src='"+this.images.handleV.src+"'>";



		//Build Controls

		this.upArrow = new DivPlus(scrollingDiv.objectID+"scrollUp",upArrowStyle);

		this.downArrow = new DivPlus(scrollingDiv.objectID+"scrollDown",downArrowStyle);

		this.channelV = new DivPlus(scrollingDiv.objectID+"channelV",channelVStyle);

		this.handleV = new DivPlus(scrollingDiv.objectID+"handleV",handleVStyle);

		

		new DivPlusDragger(this.handleV, "v", {top: 0, bottom: this.channelVFloatLimit});	

		

		

		//Methods dealing with vertical scrolling

		this.hideVertical = function(){

			this.upArrow.setDisplay("none");

			this.downArrow.setDisplay("none");

			this.channelV.setDisplay("none");

			this.channelV.setDisplay("none");

		}		

		this.showVertical = function(){

			this.upArrow.setDisplay("block");

			this.downArrow.setDisplay("block");

			this.channelV.setDisplay("block");

			this.channelV.setDisplay("block");

		}

		

		if(this.contentSize.height <= this.windowSize.height && this.images.channelV.force!="Yes"){

			this.hideVertical();

		}

		

		this.scrollUp = function(e){

			if(e || window.event) browsers.stopBubbling(e);

			if((scrollingDiv.getTop() + this.scrollSpeed < 0) && this.scrollControl){

				scrollingDiv.setTop(scrollingDiv.getTop() + this.scrollSpeed);

				var oldClip = scrollingDiv.getClipping();

				scrollingDiv.setClipping({	top:oldClip.top - this.scrollSpeed, 

											left:oldClip.left, 

											right:oldClip.right, 

											bottom:oldClip.bottom - this.scrollSpeed});

				var relativePosition = Math.abs(scrollingDiv.getTop()) / this.size.height;

				this.handleV.setTop(Math.floor(this.channelVHeight * relativePosition));

				setTimeout("divList['"+scrollingDiv.objectID+"'].scroller.scrollUp()",100);

			} else if (scrollingDiv.getTop() + this.scrollSpeed >= 0){

				this.handleV.setTop(0);

				scrollingDiv.setTop(0);

				var oldClip = scrollingDiv.getClipping();

				scrollingDiv.setClipping({	top:0, 

											left:oldClip.left, 

											right: oldClip.right, 

											bottom:this.windowSize.height});

				this.scrollControl = 0;

			}

			return false;

		}		

		this.scrollDown = function(e){

			if(e || window.event) browsers.stopBubbling(e);

			if(((scrollingDiv.getTop() + this.size.height - this.scrollSpeed) > this.windowSize.height) && this.scrollControl){

				scrollingDiv.setTop(scrollingDiv.getTop() - this.scrollSpeed);

				var oldClip = scrollingDiv.getClipping();

				scrollingDiv.setClipping({	top:oldClip.top + this.scrollSpeed, 

											left:oldClip.left, 

											right:oldClip.right, 

											bottom:oldClip.bottom + this.scrollSpeed});

				var relativePosition = Math.abs(scrollingDiv.getTop()) / this.size.height;

				this.handleV.setTop(Math.floor(this.channelVHeight * relativePosition));

				setTimeout("divList['"+scrollingDiv.objectID+"'].scroller.scrollDown()",100);

			} else if ((scrollingDiv.getTop() + this.size.height - this.scrollSpeed) <= this.windowSize.height){

				this.handleV.setTop(this.channelVFloatLimit);

				scrollingDiv.setTop(this.windowSize.height - this.size.height);

				var oldClip = scrollingDiv.getClipping();

				scrollingDiv.setClipping({	top:this.size.height - this.windowSize.height, 

											left:oldClip.left, 

											right:oldClip.right, 

											bottom:this.size.height});

				this.scrollControl = 0;			

			}

			return false;

		}

		this.dragScrollV = function(mouseY, mouseX, e){

		if(e || window.event) browsers.stopBubbling(e);

			if(this.scrollControl){

				var relativePosition = Math.abs(this.handleV.getTop() / this.channelVFloatLimit);

				var oldClip = scrollingDiv.getClipping();

				var effectiveHeight = this.size.height - this.windowSize.height;

				if(relativePosition * effectiveHeight > this.size.height){

					scrollingDiv.setTop((this.size.height - this.windowSize.height) * -1)

					scrollingDiv.setClipping({	top: this.size.height - this.windowSize.height,

												left: oldClip.left,

												right: oldClip.right,

												bottom: this.size.height});				

					} else {

					scrollingDiv.setTop(Math.floor(relativePosition * effectiveHeight * -1));

					var oldClip = scrollingDiv.getClipping();

					scrollingDiv.setClipping({	top:Math.ceil(relativePosition * effectiveHeight), 

												left:oldClip.left, 

												right:oldClip.right, 

												bottom:Math.ceil(relativePosition * effectiveHeight) + this.windowSize.height});

					}

				setTimeout("divList['"+scrollingDiv.objectID+"'].scroller.dragScrollV()", 10);

			} else {

			this.scrollControl = 0;

			}

			return false;

		}	

	}	

	

	

	this.resetSize = function(){

		this.contentSize = scrollingDiv.getSize();

		if(scrollingDiv.getWidth() + scrollingDiv.getPadding().left + scrollingDiv.getPadding().right < this.windowSize.width){

			scrollingDiv.setWidth(windowSize.width - scrollingDiv.getPadding().left - scrollingDiv.getPadding().right);

		}

		if(scrollingDiv.getHeight() + scrollingDiv.getPadding().top + scrollingDiv.getPadding().bottom < this.windowSize.height){

			scrollingDiv.setHeight(windowSize.height - scrollingDiv.getPadding().top - scrollingDiv.getPadding().bottom);

		}

	

		this.size = scrollingDiv.getSize();

		if(browsers.ns6){

			this.size.width += scrollingDiv.getPadding().left + scrollingDiv.getPadding().right;

			this.size.height += scrollingDiv.getPadding().top + scrollingDiv.getPadding().bottom;

		}

		if(defined(this.handleH)){

			this.handleHWidth = Math.floor((this.channelHWidth/ this.size.width) * this.channelHWidth);

			if(this.handleHWidth < 10) this.handleHWidth = 10;

			this.channelHFloatLimit = this.channelHWidth - this.handleHWidth;

			this.handleH.setWidth(this.handleHWidth);

			this.handleH.dragger.resetLimits({left: 0, right: this.channelHFloatLimit});

			if(this.contentSize.width<=this.windowSize.width){

				this.hideHorizontal();

			} else {

				this.showHorizontal();

			}

		}

		if(defined(this.handleV)){

			this.handleVHeight = Math.floor((this.channelVHeight/ this.size.height) * this.channelVHeight);

			if(this.handleVHeight < 10) this.handleVHeight = 10;

			this.channelVFloatLimit = this.channelVHeight - this.handleVHeight;

			this.handleV.setHeight(this.handleVHeight);

			this.handleV.dragger.resetLimits({top:0, bottom: this.channelVFloatLimit});

			if(this.contentSize.height<=this.windowSize.height){

				this.hideVertical();

			} else {

				this.showVertical();

			}

		}		

	}

	this.setHTML = function(newHTML){

		scrollingDiv.setHTML(newHTML);

		if(defined(this.handleH)) scrollingDiv.setWidth("auto");

		if(defined(this.handleV)) scrollingDiv.setHeight("auto");

		this.resetSize();

	}

}