<?PHP
//shows project stats from the pool

require_once('../pool_init.php');
require_once('db_init.php');
require_once('person.class.php');
$user = Pool_Person::cookie_auth();
if($user['permissions'] != POOL_LEVEL_ADMIN){
	echo 'You do not have sufficient rights to run this script.  Log in to the Pool as an admin.';
	die();	
}

//Project stats
echo "Project stats:<br />\n";
$q = 'SELECT `pool_type`, COUNT(*) AS number FROM `project` GROUP BY `pool_type`';
$result = $data->getAll($q, array());
check_db($result, 'pool type count');
$total = 0;
foreach($result as $type){
	echo " - $type[number] $type[pool_type] projects<br />\n";
	$total += intval($type['number']);
}
echo "For a total of $total projects.<br /><br /><hr />\n";

//artifact counts
echo "Artifact stats:<br />\n";
$q = 'SELECT `pool_type`, COUNT(*) as number, SUM(`intent_num`) AS intents, SUM(`approach_num`) AS approaches, SUM(`release_num`) AS releases FROM `project` GROUP BY `pool_type`';
$result = $data->getAll($q, array());
check_db($result, 'pool type artifact count');
$total = 0;
$intents = 0;
$approaches = 0;
$releases = 0;
foreach($result as $type){
	echo " The $type[pool_type] pool's $type[number] projects contain:<br />\n";
	echo " - $type[intents] intents<br />\n";
	echo " - $type[approaches] approaches<br />\n";
	echo " - $type[releases] releases<br />\n";		
	$total += $type['intents']; 
	$total += $type['approaches']; 
	$total += $type['releases'];
	$intents += $type['intents'];
	$approaches += intval($type['approaches']);
	$releases += $type['releases'];
}
echo "For a total of $total artifacts currently being tracked by the pool.<br /><br /><hr />\n";

//licenses
echo "License stats:<br />\n";
$q = "SELECT COUNT(*) as number, `aspect` FROM `aspects` WHERE `artifact_type` = 'approach' AND `aspect` != 'custom rights' GROUP BY `aspect`";
$result = $data->getAll($q, array());
check_db($result, 'approach license counts');
echo "Of the $approaches approaches in the pool:<br />\n";
foreach($result as $license) echo " - $license[number] have the license $license[aspect]<br />\n";
echo "<br />\n";
$q = "SELECT COUNT(*) as number, `aspect` FROM `aspects` WHERE `artifact_type` = 'release' AND `aspect` != 'custom rights' GROUP BY `aspect`";
$result = $data->getAll($q, array());
check_db($result, 'release license counts');
echo "Of the $releases releases in the pool:<br />\n";
foreach($result as $license) echo " - $license[number] have the license $license[aspect]<br />\n";

echo "<br />Unique license combinations:<br />\n";
$q = "SELECT COUNT(*) AS number, license FROM (SELECT CONCAT(`artifact_id`, `artifact_type`) AS artifact, GROUP_CONCAT(`aspect` ORDER BY `aspect`) as license FROM `aspects` WHERE `artifact_type` = 'approach' OR `artifact_type` = 'release' GROUP BY `artifact`) AS licenses GROUP BY license ORDER BY license ASC";
$result = $data->getAll($q, array());
check_db($result, 'unique licenses');
foreach($result as $license) echo " $license[license]: $license[number]<br />\n";




function check_db($result, $func){
	global $data;
	//Return true if $result is not DB::Error, false and an error otherwise
	if(DB::isError($result)){
		trigger_error("$func failed: $data->last_query \n".$result->getMessage(), E_USER_WARNING);
		die();
	} else return true;
}


?>